package com.oosic.net.connectivity;

import java.util.ArrayList;

import android.content.Context;
import android.content.IntentFilter;
import android.net.ConnectivityManager;
import android.net.NetworkInfo;
import android.net.wifi.WifiManager;

/**
 * It is the network connectivity center of the net service.
 * @author CHEN JIAN WEN
 *
 */
public class ConnectivityCenter {

    /**
     * It contains the listeners which are registered into the connectivity center.
     */
    private final ArrayList<ConnectivityListener> mListeners =
        new ArrayList<ConnectivityListener>();

    /**
     * It is the connectivity braodcast 
     */
    ConnectivityBroadcastReceiver mCBR;

    /**
     * It is the context of the application
     */
    Context mContext;

    /**
     * It constructs the connectivity center.
     * @param context It is the application context.
     */
    public ConnectivityCenter(Context context) {
        mContext = context;

        // Register the connectivity broadcast.
        mCBR = new ConnectivityBroadcastReceiver(this);

        final IntentFilter filter = new IntentFilter();
        filter.addAction(ConnectivityManager.CONNECTIVITY_ACTION);

        try {
            final String WIFI_AP_STATE_CHANGED_ACTION =
                (String) WifiManager.class.
                getField("WIFI_AP_STATE_CHANGED_ACTION").get(null);
            filter.addAction(WIFI_AP_STATE_CHANGED_ACTION);
        } catch (IllegalArgumentException e) {
            e.printStackTrace();
        } catch (IllegalAccessException e) {
            e.printStackTrace();
        } catch (SecurityException e) {
            e.printStackTrace();
        } catch (NoSuchFieldException e) {
            e.printStackTrace();
        } catch (Exception e) {
            e.printStackTrace();
        }

        try {
            final String ACTION_TETHER_STATE_CHANGED =
                (String) ConnectivityManager.class.
                getField("ACTION_TETHER_STATE_CHANGED").get(null);
            filter.addAction(ACTION_TETHER_STATE_CHANGED);
        } catch (IllegalArgumentException e) {
            e.printStackTrace();
        } catch (IllegalAccessException e) {
            e.printStackTrace();
        } catch (SecurityException e) {
            e.printStackTrace();
        } catch (NoSuchFieldException e) {
            e.printStackTrace();
        } catch (Exception e) {
            e.printStackTrace();
        }
        context.registerReceiver(mCBR, filter);
    }

    /**
     * It destroy the connectivity center.
     */
    public void destroy() {
        // Unregister the connectivity broadcast receiver.
        mContext.unregisterReceiver(mCBR);

        synchronized(mListeners) {
            mListeners.clear();
        }
    }

    /**
     * It register a listener to the connectivity center.
     * @param listener It is the listener to register.
     */
    public void registerListener(ConnectivityListener listener) {
        if (listener == null) return;

        synchronized(mListeners) {
            mListeners.add(listener);
        }
    }

    /**
     * It unregister a listener from the connectivity center.
     * @param listener It is the listener to unregister.
     */
    public void unregisterListener(ConnectivityListener listener) {
        if (listener == null) return;

        synchronized(mListeners) {
            if (mListeners.remove(listener)) {
                listener.onUnregistered();
            }
        }
    }

    /**
     * It calls the all onNoConnectivity() method of the listeners.
     */
    void onNoConnectivity() {
        synchronized(mListeners) {
            for (ConnectivityListener p : mListeners) {
                p.onNoConnectivity();
            }
        }
    }

    /**
     * It calls the all onWifiConnectivityChanged() method of the listeners.
     * @param state It is the network state.
     */
    void onWifiConnectivityChanged(NetworkInfo.State state) {
        synchronized(mListeners) {
            for (ConnectivityListener p : mListeners) {
                p.onWifiConnectivityChanged(state);
            }
        }
    }

    /**
     * It calls the all onEthernetConnectivityChanged() method of the listeners.
     * @param state It is the network state.
     */
    void onEthernetConnectivityChanged(NetworkInfo.State state) {
        synchronized(mListeners) {
            for (ConnectivityListener p : mListeners) {
                p.onEthernetConnectivityChanged(state);
            }
        }
    }

    /**
     * It calls the all onBluetoothConnectivityChanged() method of the listeners.
     * @param state It is the network state.
     */
    void onBluetoothConnectivityChanged(NetworkInfo.State state) {
        synchronized(mListeners) {
            for (ConnectivityListener p : mListeners) {
                p.onBluetoothConnectivityChanged(state);
            }
        }
    }

    /**
     * It calls the all onMobileConnectivityChanged() method of the listeners.
     * @param state It is the network state.
     * @param isRoaming It is the flag if the mobile is roaming.
     */
    void onMobileConnectivityChanged(NetworkInfo.State state, boolean isRoaming) {
        synchronized(mListeners) {
            for (ConnectivityListener p : mListeners) {
                p.onMobileConnectivityChanged(state, isRoaming);
            }
        }
    }

    /**
     * It calls the all onMobileDUNConnectivityChanged() method of the listeners.
     * @param state It is the network state.
     * @param isRoaming It is the flag if the mobile is roaming.
     */
    void onMobileDUNConnectivityChanged(NetworkInfo.State state, boolean isRoaming) {
        synchronized(mListeners) {
            for (ConnectivityListener p : mListeners) {
                p.onMobileDUNConnectivityChanged(state, isRoaming);
            }
        }
    }

    /**
     * It calls the all onMobileHIPRIConnectivityChanged() method of the listeners.
     * @param state It is the network state.
     * @param isRoaming It is the flag if the mobile is roaming.
     */
    void onMobileHIPRIConnectivityChanged(NetworkInfo.State state, boolean isRoaming) {
        synchronized(mListeners) {
            for (ConnectivityListener p : mListeners) {
                p.onMobileHIPRIConnectivityChanged(state, isRoaming);
            }
        }
    }

    /**
     * It calls the all onMobileMMSConnectivityChanged() method of the listeners.
     * @param state It is the network state.
     * @param isRoaming It is the flag if the mobile is roaming.
     */
    void onMobileMMSConnectivityChanged(NetworkInfo.State state, boolean isRoaming) {
        synchronized(mListeners) {
            for (ConnectivityListener p : mListeners) {
                p.onMobileMMSConnectivityChanged(state, isRoaming);
            }
        }
    }

    /**
     * It calls the all onMobileSUPLConnectivityChanged() method of the listeners.
     * @param state It is the network state.
     * @param isRoaming It is the flag if the mobile is roaming.
     */
    void onMobileSUPLConnectivityChanged(NetworkInfo.State state, boolean isRoaming) {
        synchronized(mListeners) {
            for (ConnectivityListener p : mListeners) {
                p.onMobileSUPLConnectivityChanged(state, isRoaming);
            }
        }
    }

    /**
     * It calls the all onWimaxConnectivityChanged() method of the listeners.
     * @param state It is the network state.
     */
    void onWimaxConnectivityChanged(NetworkInfo.State state) {
        synchronized(mListeners) {
            for (ConnectivityListener p : mListeners) {
                p.onWimaxConnectivityChanged(state);
            }
        }
    }

    /**
     * It calls the all onWifiApConnectivityChanged() method of the listeners.
     * @param state It is the network state.
     */
    void onWifiApConnectivityChanged(NetworkInfo.State state) {
        synchronized(mListeners) {
            for (ConnectivityListener p : mListeners) {
                p.onWifiApConnectivityChanged(state);
            }
        }
    }

    /**
     * It calls the all onTetherStateChanged() method of the listeners. 
     * @param extra It is the extra of active tether.
     */
    void onTetherStateChanged(ArrayList<String> extra) {
        synchronized(mListeners) {
            for (ConnectivityListener p : mListeners) {
                p.onTetherStateChanged(extra);
            }
        }
    }
}
