package com.oosic.net.wifi;

import android.net.wifi.ScanResult;
import android.net.wifi.WifiConfiguration;
import android.net.wifi.WifiInfo;
import android.net.wifi.WifiManager;
import android.util.Log;

/**
 * It is the container of wifi access point data.
 * @author CHEN JIAN WEN
 *
 */
public class AccessPoint implements Comparable<AccessPoint> {
    static final String TAG = "AccessPoint";

    /** These values are matched in string arrays -- changes must be kept in sync */
    public static final int SECURITY_NONE = 0;
    public static final int SECURITY_WEP = 1;
    public static final int SECURITY_PSK = 2;
    public static final int SECURITY_EAP = 3;

    public static final int PSK_TYPE_UNKNOWN = 200;
    public static final int PSK_TYPE_WPA = 201;
    public static final int PSK_TYPE_WPA2 = 202;
    public static final int PSK_TYPE_WPA_WPA2 = 203;


    String mSSID;
    String mBSSID;
    int mSecurity;
    int mNetworkId = -1;
    boolean mWPSAvailable = false;

    int mPskType = PSK_TYPE_UNKNOWN;

    private WifiConfiguration mConfig;
    ScanResult mScanResult;

    private int mRssi;
    private WifiInfo mInfo;

    public static int getSecurity(WifiConfiguration config) {
        if (config.allowedKeyManagement.get(KeyMgmt.WPA_PSK)) {
            return SECURITY_PSK;
        }
        if (config.allowedKeyManagement.get(KeyMgmt.WPA_EAP) ||
                config.allowedKeyManagement.get(KeyMgmt.IEEE8021X)) {
            return SECURITY_EAP;
        }
        return (config.wepKeys[0] != null) ? SECURITY_WEP : SECURITY_NONE;
    }

    public static int getSecurity(ScanResult result) {
        if (result.capabilities.contains("WEP")) {
            return SECURITY_WEP;
        } else if (result.capabilities.contains("PSK")) {
            return SECURITY_PSK;
        } else if (result.capabilities.contains("EAP")) {
            return SECURITY_EAP;
        }
        return SECURITY_NONE;
    }

    private static int getPskType(ScanResult result) {
        boolean wpa = result.capabilities.contains("WPA-PSK");
        boolean wpa2 = result.capabilities.contains("WPA2-PSK");
        if (wpa2 && wpa) {
            return PSK_TYPE_WPA_WPA2;
        } else if (wpa2) {
            return PSK_TYPE_WPA2;
        } else if (wpa) {
            return PSK_TYPE_WPA;
        } else {
            Log.w(TAG, "Received abnormal flag string: " + result.capabilities);
            return PSK_TYPE_UNKNOWN;
        }
    }

    public AccessPoint(WifiConfiguration config) {
        loadConfig(config);
    }

    public AccessPoint(ScanResult config) {
        loadResult(config);
    }

    /**
     * It combines the configure access point with scan result.
     * @param sr It is the scan result to conbime.
     * @return It returns true if the combine operation is successful. It returns false otherwise.
     */
    public boolean combineScanResults(ScanResult sr) {
        if (sr == null) return false;

        if (mSSID.equals(sr.SSID)) {
            loadResult(sr);
            return true;
        }
        return false;
    }

    private void loadConfig(WifiConfiguration config) {
        mSSID = (config.SSID == null ? "" : removeDoubleQuotes(config.SSID));
        mBSSID = config.BSSID;
        mSecurity = getSecurity(config);
        mNetworkId = config.networkId;
        mRssi = Integer.MAX_VALUE;
        mConfig = config;
    }

    private void loadResult(ScanResult result) {
        mSSID = result.SSID;
        mBSSID = result.BSSID;
        mSecurity = getSecurity(result);
        mWPSAvailable = mSecurity != SECURITY_EAP && result.capabilities.contains("WPS");
        if (mSecurity == SECURITY_PSK)
            mPskType = getPskType(result);
        mRssi = result.level;
        mScanResult = result;
    }

    int getLevel() {
        if (mRssi == Integer.MAX_VALUE) {
            return -1;
        }
        return WifiManager.calculateSignalLevel(mRssi, 4);
    }

    WifiConfiguration getConfig() {
        return mConfig;
    }

    WifiInfo getInfo() {
        return mInfo;
    }

    static String removeDoubleQuotes(String string) {
        int length = string.length();
        if ((length > 1) && (string.charAt(0) == '"')
                && (string.charAt(length - 1) == '"')) {
            return string.substring(1, length - 1);
        }
        return string;
    }

    static String convertToQuotedString(String string) {
        return "\"" + string + "\"";
    }

    /**
     * Generate and save a default wifiConfiguration with common values.
     * Can only be called for unsecured networks.
     * @hide
     */
    protected void generateOpenNetworkConfig() {
        if (mSecurity != SECURITY_NONE)
            throw new IllegalStateException();
        if (mConfig != null)
            return;
        mConfig = new WifiConfiguration();
        mConfig.SSID = AccessPoint.convertToQuotedString(mSSID);
        mConfig.allowedKeyManagement.set(KeyMgmt.NONE);
    }

    public void updateWifiInfo(WifiInfo info) {
        mInfo = info;
    }

    @Override
    public int compareTo(AccessPoint other) {
        // Active one goes first.
        if (mInfo != other.mInfo) {
            return (mInfo != null) ? -1 : 1;
        }

        // Reachable one goes before unreachable one.
        if ((mRssi ^ other.mRssi) < 0) {
            return (mRssi != Integer.MAX_VALUE) ? -1 : 1;
        }

        // Configured one goes before unconfigured one.
        if ((mNetworkId ^ other.mNetworkId) < 0) {
            return (mNetworkId != -1) ? -1 : 1;
        }

        // Sort by signal strength.
        int difference = WifiManager.compareSignalLevel(other.mRssi, mRssi);
        if (difference != 0) {
            return difference;
        }

        // Sort by ssid.
        return mSSID.compareToIgnoreCase(other.mSSID);
    }

    public static int getApSecurityTypeIndex(WifiConfiguration wifiConfig) {
        int ret = PSK_TYPE_UNKNOWN;
        if (wifiConfig.allowedKeyManagement.get(KeyMgmt.WPA_PSK)) {
            ret = PSK_TYPE_WPA;
        }
        if (wifiConfig.allowedKeyManagement.get(KeyMgmt.WPA2_PSK)) {
            if (ret == PSK_TYPE_UNKNOWN) {
                ret = PSK_TYPE_WPA2;
            } else {
                ret = PSK_TYPE_WPA_WPA2;
            }
        }
        return ret;
    }

    public static void setApSecurityTypeIndex(WifiConfiguration wifiConfig, int type) {
        if (type == SECURITY_PSK) type = PSK_TYPE_WPA;
        if (type == PSK_TYPE_WPA) {
            wifiConfig.allowedKeyManagement.set(KeyMgmt.WPA_PSK);
        } else if (type == PSK_TYPE_WPA2) {
            wifiConfig.allowedKeyManagement.set(KeyMgmt.WPA2_PSK);
        } else if (type == PSK_TYPE_WPA_WPA2) {
            wifiConfig.allowedKeyManagement.set(KeyMgmt.WPA_PSK);
            wifiConfig.allowedKeyManagement.set(KeyMgmt.WPA2_PSK);
        } else {
            wifiConfig.allowedKeyManagement.set(KeyMgmt.NONE);
        }
    }

    /**
     * It gets the SSID.
     */
    public String getSSID() {
        return mSSID;
    }

    /**
     * It gets the security.
     * @return It returns the security.
     */
    public int getSecurity() {
        return mSecurity;
    }

    /**
     * It gets the signal level in dBm.
     * @return It returns the signal level in dBm.
     */
    public int getRssi() {
        return mRssi;
    }
// +-----------------------------------------------------------------------------------------------+
// | Following code is copied from framework. DO NOT modify.                                       |
// +-----------------------------------------------------------------------------------------------+
    /**
     * Recognized key management schemes.
     */
    public static class KeyMgmt {
        public KeyMgmt() { }

        /** WPA is not used; plaintext or static WEP could be used. */
        public static final int NONE = 0;
        /** WPA pre-shared key (requires {@code preSharedKey} to be specified). */
        public static final int WPA_PSK = 1;
        /** WPA using EAP authentication. Generally used with an external authentication server. */
        public static final int WPA_EAP = 2;
        /** IEEE 802.1X using EAP authentication and (optionally) dynamically
         * generated WEP keys. */
        public static final int IEEE8021X = 3;

        /** WPA2 pre-shared key for use with soft access point
          * (requires {@code preSharedKey} to be specified).
          * @hide
          */
        public static final int WPA2_PSK = 4;
    }
}
