package com.oosic.net.wifi;


import java.util.ArrayList;
import java.util.Calendar;
import java.util.List;
import java.util.Locale;
import java.util.TimeZone;
import java.util.Timer;
import java.util.TimerTask;

import android.content.Context;
import android.content.IntentFilter;
//import android.net.ConnectivityManager;
import android.net.wifi.ScanResult;
import android.net.wifi.WifiConfiguration;
import android.net.wifi.WifiManager;
import android.os.Handler;
import android.os.Looper;

/**
 * Wifi scanner helps to scan the wifi hotspots.
 * @author CHEN JIAN WEN
 *
 */
public class WifiScanner {
    /**
     * It is the last scan time.
     */
    Calendar mLastScanTime;

    /**
     * It is the context.
     */
    Context mContext;

    /**
     * It is the data sychronize object.
     */
    Object mDataSync = new Object();

    /**
     * It is the scan results.
     */
    List<ScanResult> mScanResults;

    /**
     * It is the wifi configurations.
     */
    List<WifiConfiguration> mWifiConfigurations;

    /**
     * It is the rescan timer.
     */
    private Timer mRetryTimer;

    /**
     * It is the pause flag.
     */
    boolean mPause = false;

    /**
     * It is the scan count.
     */
    int mScanCount = 0;

    /**
     * It is the rescan task.
     */
    TimerTask mRescanTask = null;

    /**
     * It is the connectivity manager.
     */
//    private ConnectivityManager mCM;

    /**
     * It is the wifi broadcast receiver.
     */
    private ScanReceiver mWSBR;

    /**
     * It is the wifi scan complete listeners.
     */
    ArrayList<WifiScanCompleteListener> mWSCLs;

    /**
     * It constructs the wifi scanner.
     * @param context It is the context.
     */
    public WifiScanner(Context context) {
        mContext = context;
//        mCM = (ConnectivityManager)context.getSystemService(Context.CONNECTIVITY_SERVICE);
        mLastScanTime = Calendar.getInstance(TimeZone.getTimeZone("GMT+00:00"), Locale.ENGLISH);
        mLastScanTime.setTimeInMillis(0);
        mRetryTimer = null;
        mScanResults = null;
        mWifiConfigurations = null;
        mWSBR = new ScanReceiver(this);
        mWSCLs = new ArrayList<WifiScanCompleteListener>();

        final IntentFilter _if = new IntentFilter();
        _if.addAction(WifiManager.SCAN_RESULTS_AVAILABLE_ACTION);
        context.registerReceiver(mWSBR, _if);
    }

    /**
     * It destroys the wifi scaner.
     */
    public void destroy() {
        synchronized(mDataSync) {
            if (mScanResults != null) {
                mScanResults.clear();
                mScanResults = null;
            }

            if (mWifiConfigurations != null) {
                mWifiConfigurations.clear();
                mWifiConfigurations = null;
            }
        }
        synchronized(mWSCLs) {
            mWSCLs.clear();
        }
        if (mWSBR != null) {
            mContext.unregisterReceiver(mWSBR);
            mWSBR = null;
        }
    }

    /**
     * It pauses the wifi scanner.
     */
    public void pause() {
        mPause = true;
    }

    /**
     * It resumes the wifi 
     */
    public void resume() {
        mPause = false;
    }

    /**
     * It requests to scan the wifi.
     * @param forceReport It is the force report flag in positive logic which is used to make scan
     *                    complete callback runs even if the Wi-Fi state is not enalbed.
     */
    public void requestScan(boolean forceReport) {
        // Clear the retry timer.
        if (mRetryTimer != null) {
            mRetryTimer.cancel();
            mRetryTimer = null;
        }
        if (mRescanTask != null) {
            mRescanTask.cancel();
            mRescanTask = null;
        }

        final WifiManager wm = (WifiManager)mContext.getSystemService(Context.WIFI_SERVICE);

        final int state = wm.getWifiState();
        if (state == WifiManager.WIFI_STATE_ENABLING) {
            // Try again after 1 second
            mRetryTimer = new Timer();
            mRescanTask = new TimerTask() {
                @Override
                public void run() {
                    final Handler h = new Handler(Looper.getMainLooper());
                    h.post(new Runnable() {
                        @Override
                        public void run() {
                            requestScan(false);
                        }
                    });
                }
            };
            mRetryTimer.schedule(mRescanTask, 1000);
            if (forceReport) {
                synchronized(mDataSync){
                    synchronized(mWSCLs) {
                    for(WifiScanCompleteListener p : mWSCLs) {
                        p.onComplete(mScanResults, mWifiConfigurations);
                }}}
            }
            return;
        } else if (state != WifiManager.WIFI_STATE_ENABLED) {
            if (forceReport) {
                synchronized(mDataSync){
                    synchronized(mWSCLs) {
                    for(WifiScanCompleteListener p : mWSCLs) {
                        p.onComplete(mScanResults, mWifiConfigurations);
                }}}
            }
            return;
        }

        final Calendar now =
            Calendar.getInstance(TimeZone.getTimeZone("GMT+00:00"), Locale.ENGLISH);
        final long nowInM = now.getTimeInMillis();
        if (Math.abs(nowInM - mLastScanTime.getTimeInMillis()) < 1000) {
            synchronized(mDataSync){
            synchronized(mWSCLs) {
            for(WifiScanCompleteListener p : mWSCLs) {
                p.onComplete(mScanResults, mWifiConfigurations);
            }}}
            return;
        }
        mLastScanTime.setTimeInMillis(nowInM);

        mScanCount ++;
        wm.startScan();
    }

    /**
     * It register a listener to scanner.
     * @param listener It is the wifi scan complete listener to register.
     */
    public void register(WifiScanCompleteListener listener) {
        if (listener == null) return;

        synchronized(mWSCLs) {
            if (!mWSCLs.contains(listener)) {
                mWSCLs.add(listener);
            }
        }
    }

    /**
     * It unregister a listener from scanner.
     * @param listener It is the wifi scan complete listener to unregister.
     */
    public void unregister(WifiScanCompleteListener listener) {
        if (listener == null) return;

        synchronized(mWSCLs) {
            mWSCLs.remove(listener);
        }
    }
}
