package com.oozic.bitmap;

import java.io.File;

import android.graphics.Bitmap;


/**
 * It is the bitmap ASYNCHRONIZED loader. Every loader will only load one bitmap.
 * @author CHEN JIAN WEN
 *
 */
public abstract class BitmapAsyncLoader implements Runnable {

    /**
     * It is the wait signal of bitmap ASYNCHRONIZED loader.
     */
    protected static boolean mWait;

    /**
     * It is the stop signal of bitmap ASYNCHRONIZED loader.
     */
    protected static boolean mStop;

    /**
     * It is the bitmap loader thread counter.
     */
    protected static int mLoadThreadCount;

    /**
     * It initializes the static variables in the bitmap ASYNCHROIZED loader.
     */
    public static void init() {
        mWait = false;
        mStop = false;
        mLoadThreadCount = 0;
    }

    /**
     * It make a wait until all the loader stop.
     */
    public static void stopAll() {
        mWait = true;

        mStop = true;
        while (mLoadThreadCount > 0) {
            try {
                Thread.sleep(100);
            } catch (InterruptedException e) {
            }
        }

        mStop = false;
        mWait = false;
    }

    /**
     * It makes bitmap loader wait or continue.
     * @param wait If its value is true, all the loaders have to wait.
     * If its value is false, all the loaders continue to load bitmap.
     */
    public static void waitAll(boolean wait) {
        mWait = wait;  
    }

    /**
     * It is the path of the bitmap.
     */
    private String mPath;

    /**
     * It is the bitmap data set.
     */
    private BitmapDataSet mBDSet;

    /**
     * It is the index of the bitmap data in bitmap data set.
     */
    private int mIndex;

    /**
     * It is the bitmap data to store bitmap.
     */
    private BitmapData mBD;

    /**
     * It is the bitmap spy that needs to decode files.
     */
    private BitmapSpyManager mSpy;

    /**
     * It is the bitmap decode config.
     */
    private Bitmap.Config mConfig;

    /**
     * It is the keep radius of the data.
     */
    private int mKeepRadius;
    /**
     * It constructs the bitmap ASYNCHRONIZED loader.
     * @param spy It is the bitmap spy manager.
     * @param path It is the bitmap path to load.
     * @param bd It is the bitmap data.
     * @param config It is the bitmap configuration.
     */

    public BitmapAsyncLoader(BitmapSpyManager spy, String path, BitmapData bd,
            Bitmap.Config config) {
        this(spy, path, bd, config, null, 0, 0);
    }

    /**
     * It constructs the bitmap ASYNCHRONIZED loader.
     * @param spy It is the bitmap spy manager.
     * @param path It is the bitmap path to load.
     * @param bd It is the bitmap data.
     * @param config It is the bitmap configuration.
     * @param list It is the bitmap data list.
     * @param index It is the bitmap data index in the bitmap data set.
     * @param radius It is the data keep radius.
     */
    public BitmapAsyncLoader(BitmapSpyManager spy, String path, BitmapData bd,
            Bitmap.Config config, BitmapDataSet set, int index, int radius) {
        mPath = path;
        mIndex = index;
        mBD = bd;
        mSpy = spy;
        mConfig = config;
        mBDSet = set;
        mKeepRadius = radius;
    }

    /**
     * It make the loader start.
     */
    public void start() {
        if (!mBD.bUpdate) {
            mBD.bUpdate = true;
            new Thread(this).start();
        }
    }

    /**
     * This is the thread process of loading imported file thumb-nail.
     */
    @Override
    public void run() {
        mLoadThreadCount ++;
        if (mBD == null) {
            mLoadThreadCount --;
            return;
        }

        if ((mPath == null) || !isIndexInRange()) {
            mLoadThreadCount --;
            mBD.bUpdate = false;
            return;
        }

        if (mBDSet != null) {
            if (!isIndexInRange()) {
                mLoadThreadCount --;
                mBD.bUpdate = false;
                return;
            }

            while(mWait) {
                if ((Math.abs(getKeepDataCenterIndex() - mIndex) > mKeepRadius) || mStop) {
                    mLoadThreadCount --;
                    mBD.bUpdate = false;
                    return;
                }
                try {
                    Thread.sleep(500);
                } catch (InterruptedException e1) {
                }
            }
        }

        final File f = new File(mPath);
        if (!f.exists()) {
            mLoadThreadCount --;
            mBD.bUpdate = false;
            return;
        }

        if (mBDSet != null) {
            synchronized(mBDSet) {
                try {
                    final BitmapData td = mBD;
                    if (td.bmp == null) {
                        td.bmp = mSpy.decodeFile(mPath, mConfig, false, true);
                        if (td.bmp == null)
                            td.bmp = mSpy.decodeFile(mPath, mConfig, true, true);
    
                        if (td.bmp == null) {
                            mBDSet.release(false);
                            td.bmp = mSpy.decodeFile(mPath, mConfig, false, true);
                            if (td.bmp == null)
                                td.bmp = mSpy.decodeFile(mPath, mConfig, true, true);
    
                            if (td.bmp == null) {
                                mLoadThreadCount --;
                                mBD.bUpdate = false;
                                return;
                            }
                        }
                    }
                } catch (OutOfMemoryError e) {
                    mLoadThreadCount --;
                    mBD.bUpdate = false;
                    return;
                }
            }
        } else {
            try {
                final BitmapData td = mBD;
                if (td.bmp == null) {
                    td.bmp = mSpy.decodeFile(mPath, mConfig, false, true);
                    if (td.bmp == null)
                        td.bmp = mSpy.decodeFile(mPath, mConfig, true, true);

                    if (td.bmp == null) {
                        mLoadThreadCount --;
                        mBD.bUpdate = false;
                        return;
                    }
                }
            } catch (OutOfMemoryError e) {
                mLoadThreadCount --;
                mBD.bUpdate = false;
                return;
            }
        }

        mBD.bUpdate = false;
        mLoadThreadCount--;

        onDataLoaded();
    }

    /**
     * It checks if the bitmap data is in range.
     * @return It returns true if the bitmap data is in range. And it returns false if the bitmap
     * data is out of range.
     */
    private boolean isIndexInRange() {
        return (mIndex >= 0) && (mIndex < mBDSet.size());
    }

    /**
     * It is called when it loads bitmap successfully.
     */
    protected abstract void onDataLoaded();

    /**
     * It gets the center index of keep bitmap data 
     * @return
     */
    protected abstract int getKeepDataCenterIndex();
}
