package com.oozic.bitmap;

import java.io.InputStream;
import java.lang.reflect.Field;
import java.util.ArrayList;

import android.content.Context;
import android.content.res.Resources;
import android.graphics.Bitmap;
import android.graphics.Bitmap.Config;
import android.graphics.BitmapFactory;
import android.graphics.BitmapFactory.Options;
import android.graphics.Point;
import android.os.Messenger;
import android.util.Log;

import com.oozic.apps.Util;

public class BitmapSpyManager {
    /**
     * It is the class tag.
     */
    private static final String TAG = "BitmapSpy";

    /**
     * Bitmap allocate limit in VM. (4M)
     */
    private int BITMAP_SIZE_LIMIT = 1024 * 1024 * 4;

    /**
     * It is the memory limiter.
     */
    int mMemoryLimit;

    /**
     * It is the used memory size.
     */
    int mMemorySize;

    /**
     * It is the used memory size in VM.
     */
    int mMemorySizeVM;

    /**
     * It is the bitmap set that applied by bitmap spy.
     */
    ArrayList<Bitmap> mBmpSet;

    /**
     * It is the bitmap set that in VM applied by bitmap spy.
     */
    ArrayList<Bitmap> mBmpSetVM;

    /**
     * It is the context of application.
     */
    Context mContext;

    /**
     * It is the object used to synchronize.
     */
    Object mSync = new Object();

    /**
     * Messenger for communicating with service.
     */
    Messenger mService = null;

    /**
     * Flag indicating whether we have called bind on the service.
     */
    boolean mIsBound  = false;

    /**
     * It constructs the bitmap spy manager.
     * @param context It is the application context.
     */
    public BitmapSpyManager(Context context) {
        mBmpSet = new ArrayList<Bitmap>();
        mBmpSetVM = new ArrayList<Bitmap>();
        mMemorySize = 0;
        mMemorySizeVM = 0;
        mMemoryLimit = 32 * 1024 * 1024;

        mContext = context;

        // Create and clean the folder spy temporary folder.
        Util.cleanupFolder(com.oozic.apps.Settings.SPY_TEMP_PATH, null);

        //doBindService();
    }

    /**
     * It reports the information of spied bitmap.
     * @param detail If its value is true, it will log all information of spied bitmap.
     * If its value is false,  it only log the number and size of the spied bitmap.
     */
    public void report(boolean detail) {
        if (!detail) {
            Log.i(TAG, String.format(
                    "Size --%d--|--%d-- Limit --%d--", mMemorySizeVM, mMemorySize, mMemoryLimit));
            return;
        }

        int i = 0;
        synchronized(mSync) {
            Log.i(TAG, "Memory used out of the VM.");
            for (Bitmap p : mBmpSet) {
                if (p != null) {
                    Log.i(TAG, String.format("Bitmap --%d--", i));
                    Log.i(TAG, String.format("Width = %d", p.getWidth()));
                    Log.i(TAG, String.format("Height = %d", p.getHeight()));
                    Log.i(TAG, String.format("Size = %d", p.getWidth() * p.getHeight() * getPixelSize(p.getConfig())));
                }
                i ++;
            }
    
            Log.i(TAG, "Memory used in the VM.");
            for (Bitmap p : mBmpSetVM) {
                if (p != null) {
                    Log.i(TAG, String.format("Bitmap --%d--", i));
                    Log.i(TAG, String.format("Width = %d", p.getWidth()));
                    Log.i(TAG, String.format("Height = %d", p.getHeight()));
                    Log.i(TAG, String.format("Size = %d", p.getWidth() * p.getHeight() * getPixelSize(p.getConfig())));
                }
                i ++;
            }
        }

        Log.i(TAG, String.format(
                "Totle --%d-- Size --%d:%d--|--%d:%d-- Limit --%d--", i,
                mMemorySizeVM, mMemorySizeVM * 100 / mMemoryLimit,
                mMemorySize, mMemorySize * 100 / mMemoryLimit, mMemoryLimit));
        
    }

    /**
     * It prepares the special option to get the memory out of VM.
     * It is only useful before ANDROID 4.0.
     * @return It returns a decode options object. If it returns null value, that's impossible,
     * this means ANDROID API does not support the option setting.
     */
    @SuppressWarnings("rawtypes")
    private Options prepareSpyOptions() {
        Options opt = new Options();
        Class ownerClass = opt.getClass();
        try {
            Field inNativeAlloc = ownerClass.getField("inNativeAlloc");
            inNativeAlloc.setBoolean(opt, true);
        } catch (IllegalArgumentException e) {
            return null;
        } catch (IllegalAccessException e) {
            return null;  
        } catch (SecurityException e) {
            return null;
        } catch (NoSuchFieldException e) {
            return null;
        }
        return opt;
    }

    /**
     * It prepare the options to decode bitmap.
     * @param dp It is the decode parameters.
     * @return It returns a option that used by decode process.
     */
    private Options prepareOptions(DecodeParameters dp) {
        Options opt = null;
        if (!dp.mInVM) {
            opt = prepareSpyOptions();
        }

        final int width = dp.mSize.x;
        final int height = dp.mSize.y;
        final int total = width * height * getPixelSize(dp.mConfig);

        if ((opt == null) || dp.mLimited) {
            if ((opt == null) || dp.mInVM) {
                opt = new Options();
                dp.mInVM = true;
            }
            if (total > BITMAP_SIZE_LIMIT) {
                final float scale = 1.0f / dp.mScaleRate;
                float s = Math.max((float)(total / BITMAP_SIZE_LIMIT), scale);
                int n = (int)(s + 0.5f);
                n = Util.nextPowerOf2(n);
                opt.inDither = true;
                opt.inSampleSize = n;
            } else {
                if (dp.mScaleRate != 1.0f) {
                    final float s = 1.0f / dp.mScaleRate;
                    int n = (int)(s + 0.5f);
                    n = Util.nextPowerOf2(n);
                    opt.inDither = true;
                    opt.inSampleSize = n;
                }
            }
        } else {
            if (dp.mScaleRate != 1.0f) {
                final float s = 1.0f / dp.mScaleRate;
                int n = (int)(s + 0.5f);
                n = Util.nextPowerOf2(n);
                opt.inDither = true;
                opt.inSampleSize = n;
            }
        }
        opt.inPreferredConfig = dp.mConfig;
        return opt;
    }

    /**
     * It gets the bitmap size of a bitmap.
     * @param outSize It is the out buffer of bitmap size.
     * @param res It is the resource contains the bitmap to decode.
     * @param index It is the resource bitmap id.
     */
    public static void getBitmapSize(Point outSize, Resources res, int index) {
        if ((outSize == null) || (res == null)) return;

        Options opt = new Options();
        opt.inJustDecodeBounds = true;
        BitmapFactory.decodeResource(res, index, opt);

        outSize.set(opt.outWidth, opt.outHeight);
    }

    /**
     * It gets the bitmap size of a bitmap.
     * @param outSize It is the out buffer of bitmap size.
     * @param data It is byte buffer contains the bitmap to decode.
     * @param offset It is byte buffer index of valid data.
     * @param length It is byte buffer length of valid data.
     */
    public static void getBitmapSize(Point outSize, byte data[], int offset, int length) {
        if ((outSize == null) || (data == null)) return;

        Options opt = new Options();
        opt.inJustDecodeBounds = true;
        BitmapFactory.decodeByteArray(data, offset, length, opt);

        outSize.set(opt.outWidth, opt.outHeight);
    }

    /**
     * It gets the bitmap size of a bitmap.
     * @param outSize It is the out buffer of bitmap size.
     * @param path It is the bitmap file path.
     */
    public static void getBitmapSize(Point outSize, String path) {
        if ((outSize == null) || (path == null)) return;

        Options opt = new Options();
        opt.inJustDecodeBounds = true;
        BitmapFactory.decodeFile(path, opt);

        outSize.set(opt.outWidth, opt.outHeight);
    }

    /**
     * It gets the bitmap size of a bitmap.
     * @param outSize It is the out buffer of bitmap size.
     * @param path It is the bitmap file path.
     */
    public static void getBitmapSize(Point outSize, InputStream is) {
        if ((outSize == null) || (is == null)) return;

        Options opt = new Options();
        opt.inJustDecodeBounds = true;
        BitmapFactory.decodeStream(is, null, opt);

        outSize.set(opt.outWidth, opt.outHeight);
    }

    /**
     * It create a mutable bitmap with special size and configure.
     * @param width It is the width of the bitmap to create.
     * @param height It is the height of the bitmap to create.
     * @param config It is the bitmap configure.
     * @return It returns a mutable bitmap or returns null if it is failed to create the bitmap.
     */
    public Bitmap createBitmap(int width, int height, Config config) {
        Bitmap bmp;
        try {
            bmp = Bitmap.createBitmap(width, height, config);
        } catch (OutOfMemoryError e) {
            Log.w(TAG, e.toString());
            bmp = null;
        } catch (IllegalArgumentException e) {
            Log.w(TAG, e.toString());
            bmp = null;
        }

        attach(bmp, true);
        return bmp;
    }

    /**
     * It decodes a resource to bitmap.
     * @param res It is the resources handle.
     * @param index It is the index of resource.
     * @param config It is the bitmap configure.
     * @param inVM It is the flag decides if it is going to decode bitmap in VM memory. Note, it may
     * decodes the bitmap in VM memory even if it set to false when the device did not support to
     * decode bitmap out of VM memory.
     * @param limited If this parameter is true, the bitmap will be decoded with zooming into limit
     * size event if the inVM parameter is set to true.
     * @return It returns a bitmap if it is successful. It returns null when there is not enough
     * memory.
     */
    public Bitmap decodeResouce(Resources res, int index, Bitmap.Config config, boolean inVM, boolean limited) {
        return decodeResouce(res, index, config, 1.0f, inVM, limited);
    }

    /**
     * It decodes a resource to bitmap.
     * @param res It is the resources handle.
     * @param index It is the index of resource.
     * @param config It is the bitmap configure.
     * @param scaleRate It is the scale rate of the bitmap to decode to. Its value is from 0 to 1.
     * Any value lager than 1 is equals to 1. And 0 value or the value less than 0 will cause null
     * point return value.
     * @param inVM It is the flag decides if it is going to decode bitmap in VM memory. Note, it may
     * decodes the bitmap in VM memory even if it set to false when the device did not support to
     * decode bitmap out of VM memory.
     * @param limited If this parameter is true, the bitmap will be decoded with zooming into limit
     * size event if the inVM parameter is set to true.
     * @return It returns a bitmap if it is successful. It returns null when there is not enough
     * memory.
     */
    public Bitmap decodeResouce(Resources res, int index, Bitmap.Config config, float scaleRate, boolean inVM, boolean limited) {
        if (res == null) return null;

        if (scaleRate <= 0) return null; 

        if (!inVM && mMemorySize >= mMemoryLimit) {
            Log.w(TAG, "Out of the limited memory! Cannot decode resource " + String.valueOf(index));
            report(false);
            return null;
        }

        DecodeParameters dp = new DecodeParameters(config, scaleRate, inVM, limited);
        getBitmapSize(dp.mSize, res, index);
        Options opt = prepareOptions(dp);
        Bitmap bmp;
        try {
            bmp = BitmapFactory.decodeResource(res, index, opt);
        } catch (OutOfMemoryError e) {
            report(false);
            bmp = null;
            Log.w(TAG, "Not enough memory! Fail to decode resource " + index);
        }

        attach(bmp, dp.mInVM);

        return bmp;
    }

    /**
     * It decodes a byte array to bitmap.
     * @param data It is the byte array.
     * @param offset It is the offset of valid data.
     * @param length It is the length of valid data.
     * @param config It is the bitmap configure.
     * @param inVM It is the flag decides if it is going to decode bitmap in VM memory. Note, it may
     * decodes the bitmap in VM memory even if it set to false when the device did not support to
     * decode bitmap out of VM memory.
     * @param limited If this parameter is true, the bitmap will be decoded with zooming into limit
     * size event if the inVM parameter is set to true.
     * @return It returns a bitmap if it is successful. It returns null when there is not enough
     * memory.
     */
    public Bitmap decodeByteArray(byte[] data, int offset, int length, Bitmap.Config config, boolean inVM, boolean limited) {
        return decodeByteArray(data, offset, length, config, 1.0f, inVM, limited);
    }

    /**
     * It decodes a byte array to bitmap.
     * @param data It is the byte array.
     * @param offset It is the offset of valid data.
     * @param length It is the length of valid data.
     * @param config It is the bitmap configure.
     * @param scaleRate It is the scale rate of the bitmap to decode to. Its value is from 0 to 1.
     * Any value lager than 1 is equals to 1. And 0 value or the value less than 0 will cause null
     * point return value.
     * @param inVM It is the flag decides if it is going to decode bitmap in VM memory. Note, it may
     * decodes the bitmap in VM memory even if it set to false when the device did not support to
     * decode bitmap out of VM memory.
     * @param limited If this parameter is true, the bitmap will be decoded with zooming into limit
     * size event if the inVM parameter is set to true.
     * @return It returns a bitmap if it is successful. It returns null when there is not enough
     * memory.
     */
    public Bitmap decodeByteArray(byte[] data, int offset, int length, Bitmap.Config config, float scaleRate, boolean inVM, boolean limited) {
        if (data == null) return null;

        if (scaleRate <= 0) return null; 

        if (!inVM && mMemorySize >= mMemoryLimit) {
            Log.w(TAG, "Out of the limited memory! Cannot decode byte array!");
            report(false);
            return null;
        }

        DecodeParameters dp = new DecodeParameters(config, scaleRate, inVM, limited);
        getBitmapSize(dp.mSize, data, offset, length);
        Options opt = prepareOptions(dp);
        Bitmap bmp;
        try {
            bmp = BitmapFactory.decodeByteArray(data, offset, length, opt);
        } catch (OutOfMemoryError e) {
            report(false);
            bmp = null;
            Log.w(TAG, "Not enough memory! Fail to decode byte array!");
        }

        attach(bmp, dp.mInVM);

        return bmp;
    }

    /**
     * It decodes a file to bitmap.
     * @param path It is the file path of the .
     * @param index It is the index of resource.
     * @param config It is the bitmap configure.
     * @param inVM It is the flag decides if it is going to decode bitmap in VM memory. Note, it may
     * decodes the bitmap in VM memory even if it set to false when the device did not support to
     * decode bitmap out of VM memory.
     * @param limited If this parameter is true, the bitmap will be decoded with zooming into limit
     * size event if the inVM parameter is set to true.
     * @return It returns a bitmap if it is successful. It returns null when there is not enough
     * memory.
     */
    public Bitmap decodeFile(String path, Bitmap.Config config, boolean inVM, boolean limited) {
        return decodeFile(path, config, 1.0f, inVM, limited);
    }

    /**
     * It decodes a file to bitmap.
     * @param path It is the file path of the .
     * @param index It is the index of resource.
     * @param config It is the bitmap configure.
     * @param scaleRate It is the scale rate of the bitmap to decode to. Its value is from 0 to 1.
     * Any value lager than 1 is equals to 1. And 0 value or the value less than 0 will cause null
     * point return value.
     * @param inVM It is the flag decides if it is going to decode bitmap in VM memory. Note, it may
     * decodes the bitmap in VM memory even if it set to false when the device did not support to
     * decode bitmap out of VM memory.
     * @param limited If this parameter is true, the bitmap will be decoded with zooming into limit
     * size event if the inVM parameter is set to true.
     * @return It returns a bitmap if it is successful. It returns null when there is not enough
     * memory.
     */
    public Bitmap decodeFile(String path, Bitmap.Config config, float scaleRate, boolean inVM, boolean limited) {
        if (path == null) return null;

        if (scaleRate <= 0) return null;

        if (!inVM && mMemorySize >= mMemoryLimit) {
            Log.w(TAG, "Out of the limited memory! Cannot decode file " + path);
            report(false);
            return null;
        }

        DecodeParameters dp = new DecodeParameters(config, scaleRate, inVM, limited);
        getBitmapSize(dp.mSize, path);
        Options opt = prepareOptions(dp);
        Bitmap bmp;
        try {
            bmp = BitmapFactory.decodeFile(path, opt);
        } catch (OutOfMemoryError e) {
            report(false);
            bmp = null;
            Log.w(TAG, "Not enough memory! Fail to decode" + path);
        }

        attach(bmp, dp.mInVM);

        return bmp;
    }

    /**
     * It decodes a stream to bitmap.
     * @param path It is the file path of the .
     * @param index It is the index of resource.
     * @param config It is the bitmap configure.
     * @param inVM It is the flag decides if it is going to decode bitmap in VM memory. Note, it may
     * decodes the bitmap in VM memory even if it set to false when the device did not support to
     * decode bitmap out of VM memory.
     * @param limited If this parameter is true, the bitmap will be decoded with zooming into limit
     * size event if the inVM parameter is set to true.
     * @return It returns a bitmap if it is successful. It returns null when there is not enough
     * memory.
     */
    public Bitmap decodeStream(InputStream is, Bitmap.Config config, boolean inVM, boolean limited) {
        return decodeStream(is, config, 1.0f, inVM, limited);
    }

    /**
     * It decodes a stream to bitmap.
     * @param path It is the file path of the .
     * @param index It is the index of resource.
     * @param config It is the bitmap configure.
     * @param scaleRate It is the scale rate of the bitmap to decode to. Its value is from 0 to 1.
     * Any value lager than 1 is equals to 1. And 0 value or the value less than 0 will cause null
     * point return value.
     * @param inVM It is the flag decides if it is going to decode bitmap in VM memory. Note, it may
     * decodes the bitmap in VM memory even if it set to false when the device did not support to
     * decode bitmap out of VM memory.
     * @param limited If this parameter is true, the bitmap will be decoded with zooming into limit
     * size event if the inVM parameter is set to true.
     * @return It returns a bitmap if it is successful. It returns null when there is not enough
     * memory.
     */
    public Bitmap decodeStream(InputStream is, Bitmap.Config config, float scaleRate, boolean inVM, boolean limited) {
        if (is == null) return null;
        
        if (scaleRate <= 0) return null;

        if (!inVM && mMemorySize >= mMemoryLimit) {
            Log.w(TAG, "Out of the limited memory! Cannot decode stream ");
            report(false);
            return null;
        }

        DecodeParameters dp = new DecodeParameters(config, scaleRate, inVM, limited);
        getBitmapSize(dp.mSize, is);
        Options opt = prepareOptions(dp);
        Bitmap bmp;
        try {
            bmp = BitmapFactory.decodeStream(is, null, opt);
        } catch (OutOfMemoryError e) {
            report(false);
            bmp = null;
            Log.w(TAG, "Not enough memory! Fail to decode");
        }

        attach(bmp, dp.mInVM);

        return bmp;
    }

    /**
     * It is used to get the memory use of every pixel.
     * @param config It is the bitmap configuration.
     * @return It returns the memory use of every pixel.
     */
    public static int getPixelSize(Config config) {
        int ret = 0;
        if (config == Config.ARGB_4444) ret = 2;
        else if (config == Config.ALPHA_8) ret = 1;
        else if (config == Config.ARGB_8888) ret = 4;
        else if (config == Config.RGB_565) ret = 2;
        return ret;
    }

    /**
     * It recycle the bitmap.
     * @param bmp It is the bitmap to recycle.
     */
    public void recycle(Bitmap bmp) {
        if (bmp == null) return;
        final int size = bmp.getWidth() * bmp.getHeight() * getPixelSize(bmp.getConfig());
        synchronized(mSync) {
            if (mBmpSet.indexOf(bmp) < 0) {
                if (mBmpSetVM.indexOf(bmp) >= 0) {
                    mBmpSetVM.remove(bmp);
                    mMemorySizeVM -= size;
                }
            } else {
                mBmpSet.remove(bmp);
                mMemorySize -= size;
            }
        }
        if (!bmp.isRecycled()) bmp.recycle();
    }

    /**
     * It attachs a bitmap that does not created by bitmap spy manager to spy manager.
     * <BR>
     * When you use this method, please make sure the bitmap set does not contain the
     * the bitmap to attach. You can call hasBitmap() to check.
     * @param bmp It is the bitmap to attach.
     * @param inVM It is the flag indicates if the bitmap's memeory is allocated in VM.
     */
    public void attach(Bitmap bmp, boolean inVM) {
        if (bmp != null) {
            synchronized(mSync) {
                if (!inVM) {
                    mMemorySize += bmp.getWidth() * bmp.getHeight() * getPixelSize(bmp.getConfig());
                    mBmpSet.add(bmp);
                } else {
                    mMemorySizeVM += bmp.getWidth() * bmp.getHeight() * getPixelSize(bmp.getConfig());
                    mBmpSetVM.add(bmp);
                }
            }
        }
    }

    /**
     * It checks if the bitmap already exists in the bitmap set.
     * @param bmp It is the bitmap to check.
     * @return It returns true if the bitmap is contianed in the bitmap set, and return false
     * otherwise.
     */
    public boolean hasBitmap(Bitmap bmp) {
        if (bmp != null) {
            synchronized(mSync) {
                for (Bitmap p : mBmpSet) {
                    if (p == bmp) return true;
                }
                for (Bitmap p : mBmpSetVM) {
                    if (p == bmp) return true;
                }
            }
        }
        return false;
    }


    /**
     * It destroy the spy bitmap manager.
     */
    public void destroy() {
        synchronized(mSync) {
            for (Bitmap p : mBmpSet) {
                mMemorySize -= p.getWidth() * p.getHeight() * getPixelSize(p.getConfig());
                if (!p.isRecycled()) p.recycle();
            }
            mBmpSet.clear();
    
            for (Bitmap p : mBmpSetVM) {
                mMemorySizeVM -= p.getWidth() * p.getHeight() * getPixelSize(p.getConfig());
                if (!p.isRecycled()) p.recycle();
            }
            mBmpSetVM.clear();
        }

        if ((mMemorySize != 0) || (mMemorySizeVM != 0)) {
            report(false);
            Log.w(TAG, "It may have memory leak!!!");
        }

        // Clean the temporary folder.
        Util.cleanupFolder(com.oozic.apps.Settings.SPY_TEMP_PATH, null);
    }

    /**
     * It contains the decode parameters that used in prepareOptions().
     * @author CHEN JIAN WEN
     *
     */
    class DecodeParameters {
        /**
         * It decides if the decode process uses the memory in the virtual machine.
         */
        boolean mInVM;

        /**
         * It decides if the bitmap size is limited.
         */
        boolean mLimited;

        /**
         * It is the decode configuration.
         * @see Config
         */
        Config mConfig;

        /**
         * It is the size of the bitmap.
         */
        Point mSize;

        /**
         * It is the scale rate of the bitmap.
         */
        float mScaleRate;

        /**
         * It constructs the decode parameters.
         * @param config It is the bitmap decode configuration.
         * @param scaleRate it is the bitmap scale rate. It's range is from 0 to 1.
         * @param inVM It is the flag decides if it is going to decode bitmap in VM memory. Note, it may
         * decodes the bitmap in VM memory even if it set to false when the device did not support to
         * decode bitmap out of VM memory.
         * @param limited If this parameter is true, the bitmap will be decoded with zooming into limit
         * size event if the inVM parameter is set to true.
         */
        DecodeParameters(Config config, float scaleRate, boolean inVM, boolean limited) {
            mSize = new Point();
            mConfig = config;
            mInVM = inVM;
            mLimited = limited;
            if (scaleRate < 0.0f) scaleRate = 0.0f;
            else if (scaleRate > 1.0f) scaleRate = 1.0f;
            mScaleRate = scaleRate;
        }
    }

    /**
     * It sets the limit of bitmap size. It is useful when you decode a bitmap with size limit.
     * @param limit It is the limit size of bitmap in byte unit. If this value is less than 4096,
     * the limit size will be set to 4096.
     */
    public void setBitmapSizeLimit(int limit) {
        if (limit < 4096) limit = 4096;
        BITMAP_SIZE_LIMIT = limit;
    }
}
