package com.oozic.designpatterns;

import java.util.ArrayList;

/**
 * It is the controller information manager to manage the information of the controllers.
 * @author CHEN JIAN WEN
 *
 */
public class ControllerInfoManager {
    /**
     * It is the controller manager buffer.
     */
    protected ArrayList<ControllerInfo> mControllerInfos;

    /**
     * It constructs the controller manager.
     */
    public ControllerInfoManager() {
        mControllerInfos = new ArrayList<ControllerInfo>();
    }

    /**
     * It adds a controller information to controller information manager.
     * @param name It is the name of the controller.
     * @param controller It is the controller.
     */
    public void add(String name, ControllerActions controller) {
        if ((name == null) || (controller == null)) return;

        final ControllerInfo info = new ControllerInfo(name, controller);

        synchronized(mControllerInfos) {
            final int n = mControllerInfos.size();
            int i;
            for (i = n - 1; i >= 0; i--) {
                if (info.mName.equals(mControllerInfos.get(i).mName)) {
                    break;
                }
            }
            if (i == -1) {
                mControllerInfos.add(info);
            }
        }
    }

    /**
     * It removes a controller information from controller information manager.
     * @param name It is the name of controller to remove.
     */
    public void remove(String name) {
        if (name == null) return;

        ControllerInfo rInfo = null;

        synchronized(mControllerInfos) {
            for (ControllerInfo info : mControllerInfos) {
                if (info.mName.equals(name)) {
                    rInfo = info;
                    mControllerInfos.remove(info);
                }
            }
        }

        if (rInfo != null) {
            rInfo.mController.destroy();
            rInfo.mController = null;
            rInfo.mName = null;
        }
    }

    /**
     * It helps to get a controller in the controller information manager.
     * @return It returns a controller.
     */
    public ControllerActions get(String name) {
        if (name == null) return null;

        synchronized(mControllerInfos) {
            for (ControllerInfo info : mControllerInfos) {
                if (info.mName.equals(name)) {
                    return info.mController;
                }
            }
        }
        return null;
    }

    /**
     * It removes all controller information from controller information manager.
     */
    protected void removeAll() {
        final ArrayList<ControllerInfo> bkInfos = new ArrayList<ControllerInfo>();
        synchronized(mControllerInfos) {
            for (ControllerInfo info : mControllerInfos) {
                bkInfos.add(info);
            }
            mControllerInfos.clear();
        }
        for (ControllerInfo info : bkInfos) {
            info.mController.destroy();
            info.mController = null;
            info.mName = null;
        }
        bkInfos.clear();
    }

    /**
     * It resumes all the controllers.
     */
    public void resume() {
        synchronized(mControllerInfos) {
            for (ControllerInfo info : mControllerInfos) {
                info.mController.resume();
            }
        }
    }

    /**
     * It pauses all the controllers.
     */
    public void pause() {
        synchronized(mControllerInfos) {
            for (ControllerInfo info : mControllerInfos) {
                info.mController.pause();
            }
        }
    }
}
