package com.oozic.net;

import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.IOException;



/**
 * A DataPackage grouped the different type of data to a whole package.
 * <p>
 * All the data should be send to the remote devices by a special
 * sequence(protocol), and the remote device need the special logic to receive
 * the information from the sender. So the DataPackage is designed to store, and
 * send/receive the data to the remote devices, no matter the remote devices is
 * running on Android, IOS, or win-mobile.
 * 
 * @author LIU GUO PING & CHEN JIAN WEN
 * 
 */
public class DataPackage {

    /**
     * It is the package header size.
     */
    static final int PKG_HEADER_SIZE = 20;

    /**
     * DataPackage type: None, Unknown.
     */
    public static final int PKG_TYPE_NONE = 0;

    /**
     * DataPackage type: byte[].
     */
    public static final int PKG_TYPE_BYTE_BUFFER = 1;

    /**
     * DataPackage type: String[].
     */
    public static final int PKG_TYPE_STRING = 2;

    /**
     * It is the key event package type.
     */
    public static final int PKG_TYPE_KEY_EVENT = 3;

    /**
     * It is the motion event package type.
     */
    public static final int PKG_TYPE_MOTION_EVENT = 4;

    /**
     * It is the sensor event package type.
     */
    public static final int PKG_TYPE_SENSOR_EVENT = 5;

    /**
     * It is the motion event package type.
     */
    public static final int PKG_TYPE_FULL_MOTION_EVENT = 6;

    static final int PKG_TYPE_SYSTEM = 0x1000;

    /**
     * DataPackage type: Custom DataPackage type begin.
     */
    public static final int PKG_TYPE_USER = 0x8000;

    // ------------------------------------------------------------------------

    /**
     * DataPackage action: unknown.
     */
    public static final int ACTION_NONE = 0x77770000;

    /**
     * DataPackage action: View.
     */
    public static final int ACTION_VIEW = 0x77770001;

    /**
     * DataPackage action: Inform.
     */
    public static final int ACTION_INFORM = 0x77770002;

    /**
     * DataPackage action: Query.
     */
    public static final int ACTION_QUERY = 0x77770003;

    /**
     * DataPackage action: reply.
     */
    public static final int ACTION_REPLY = 0x77770004;

    /**
     * DataPackage action: request.
     */
    public static final int ACTION_REQUEST = 0x77770005;

    /**
     * DataPackage action: pause.
     */
    public static final int ACTION_PAUSE = 0x77770006;

    /**
     * DataPackage action: resume.
     */
    public static final int ACTION_RESUME = 0x77770007;

    /**
     * DataPackage action: accept.
     */
    public static final int ACTION_ACCEPT = 0x77770008;

    /**
     * DataPackage action: cancel.
     */
    public static final int ACTION_CANCEL = 0x77770009;

    /**
     * DataPackage action: reject.
     */
    public static final int ACTION_REJECT = 0x7777000A;

    /**
     * DataPackage action: add.
     */
    public static final int ACTION_ADD = 0x7777000B;

    /**
     * DataPackage action: remove.
     */
    public static final int ACTION_REMOVE = 0x7777000C;

    /**
     * DataPackage action: update.
     */
    public static final int ACTION_UPDATE = 0x7777000D;

    /**
     * DataPackage action: View.
     */
    public static final int ACTION_AIRVIEW = 0x7777000E;
    
    /**
     * DataPackage action: Call.
     */
    public static final int ACTION_CALL = 0x7777000F;

    /**
     * It is the JSON action to transfer json data.
     */
    public static final int ACTION_JSON = 0x77770010;

    /**
     * DataPackage type.Key member of the DataPackage.
     */
    protected int mType = PKG_TYPE_NONE;

    /**
     * Unique identify of the DataPackage.
     */
    protected int mUID = 0;

    /**
     * DataPackage action type.
     */
    protected int mAction = ACTION_NONE;

    /**
     * The Package sending sequence id.
     */
    protected int mPkgID = 0;

    /**
     * The Package checksum value.
     */
    protected int mCheckValue;

    public DataPackage() {

    }

    /**
     * Get the DataPackage type.
     * 
     * @return the DataPackage type.
     * 
     * @see #PKG_TYPE_BYTE_BUFFER
     * @see #PKG_TYPE_STRING
     */
    public int getType() {
        return mType;
    }

    /**
     * Set the DataPackage action.
     * 
     * @param action
     *            the new DataPackage action.
     * @see #getAction()
     */
    public void setAction(int action) {
        mAction = action;
    }

    /**
     * Get the DataPackage action type.
     * 
     * @return the DataPackage action type.
     * @see #setAction(int)
     */
    public int getAction() {
        return mAction;
    }

    /**
     * Set the DataPackage unique identify.
     * <p>
     * The DataPackage unique identify is used when informing the DataPackage is
     * sent. If the unique identify is not zero, a notification will be sent.
     * 
     * @param id
     *            the DataPackage unique identify.
     * @see OnDataSendListener#onSend(ip, uid)
     */
    public void setID(int id) {
        mUID = id;
    }

    /**
     * Get the DataPackage unique identify.
     * 
     * @return the DataPackage unique identify.
     * @see #setID(int)
     */
    public int getID() {
        return mUID;
    }

    /**
     * Set the DataPackage sending sequence.
     * 
     * @param id
     */
    protected void setPackageID(int id) {
        mPkgID = id;
        mCheckValue = mAction + mPkgID;
    }

    protected boolean isValidAction(int action) {
        if (action >= ACTION_NONE && action <= ACTION_CANCEL) {
            if (action + mPkgID == mCheckValue) {
                return true;
            }
        }

        return false;
    }

    // ---------------------------------------- ><8 ---------------------------

    /**
     * Write the DataPackage content to the socket stream.
     * 
     * @param dos
     *            The data output Stream.
     * @throws IOException
     */
    protected void write(DataOutputStream dos) throws IOException {
    }

    /**
     * Read the DataPackage content from the socket stream.
     * 
     * @param dos
     *            The data input Stream.
     * @throws IOException
     */
    protected void read(DataInputStream dis) throws IOException {
    }

    // ---------------------------------------- ><8 ---------------------------

    /**
     * Helper function: Write the DataPackage header to socket stream.
     * 
     * @param out
     *            The data output Stream.
     * @throws IOException
     */
    protected void writePkgHeader(DataOutputStream out) throws IOException {
        out.writeInt(mType);
        out.writeInt(mUID);

        out.writeInt(mAction);
        out.writeInt(mPkgID);
        out.writeInt(mCheckValue);
    }

    /**
     * It writes the package header to byte array.
     * @param out It is the byte array to write.
     * @return It returns the package header length.
     */
    public int writePkgHeader(byte[] out) {
        final byte btype[] = Utils.intToByteArray(mType);
        final byte buid[] = Utils.intToByteArray(mUID);
        final byte baction[] = Utils.intToByteArray(mAction);
        final byte bpkgid[] = Utils.intToByteArray(mPkgID);
        final byte bcv[] = Utils.intToByteArray(mCheckValue);
        int i = 0;
        System.arraycopy(btype, 0, out , i , 4);
        i += 4;
        System.arraycopy(buid, 0, out, i, 4);
        i += 4;
        System.arraycopy(baction, 0, out, i, 4);
        i += 4;
        System.arraycopy(bpkgid, 0, out, i, 4);
        i += 4;
        System.arraycopy(bcv, 0, out, i, 4);
        i += 4;
        return i;
    }

    /**
     * Helper function: Read the DataPackage header from socket stream.
     * 
     * @param in
     *            The data input Stream.
     * @throws IOException
     */
    protected void readPkgHeader(DataInputStream in) throws IOException {
        mUID = in.readInt();
        mAction = in.readInt();

        mPkgID = in.readInt();
        mCheckValue = in.readInt();
    }

    /**
     * It reads the packge type. 
     * @param in It is the byte array to read.
     * @return It returns the package type.
     */
    public int readPkgType(byte[] in) {
        int i = 0;
        byte b[] = new byte[4];
        System.arraycopy(in, i, b, 0, 4);
        mType = Utils.byteArrayToInt(b);
        return mType;
    }

    /**
     * It read the package header.
     * @param in It is the byte array to read.
     */
    public void readPkgHeader(byte[] in) {
        int i = 0;
        byte b[] = new byte[4];
        System.arraycopy(in, i, b, 0, 4);
        mType = Utils.byteArrayToInt(b);
        i += 4;

        System.arraycopy(in, i, b, 0, 4);
        mUID = Utils.byteArrayToInt(b);
        i += 4;

        System.arraycopy(in, i, b, 0, 4);
        mAction = Utils.byteArrayToInt(b);
        i += 4;

        System.arraycopy(in, i, b, 0, 4);
        mPkgID = Utils.byteArrayToInt(b);
        i += 4;

        System.arraycopy(in, i, b, 0, 4);
        mCheckValue = Utils.byteArrayToInt(b);
    }

    /**
     * 
     * @param buffer
     * @throws UnsupportedOperationException
     *             Every time the method is invoked.
     */
    public void setByteBuffer(byte[] buffer) {
        throw new UnsupportedOperationException("setByteBuffer(byte[] buffer)");
    }

    /**
     * 
     * @return Buffer array content of the Package.
     * @throws UnsupportedOperationException
     *             Every time the method is invoked.
     */
    public byte[] getByteBuffer() {
        throw new UnsupportedOperationException("getByteBuffer()");
    }

    // ---------------------------------------- ><8 ---------------------------

    /**
     * 
     * @param string
     * @throws UnsupportedOperationException
     *             Every time the method is invoked.
     */
    public void setString(String string) {
        throw new UnsupportedOperationException("setString(String string)");
    }

    /**
     * 
     * @return String content of the Package.
     * @throws UnsupportedOperationException
     *             Every time the method is invoked.
     */
    public String getString() {
        throw new UnsupportedOperationException("getString()");
    }
}