package com.oozic.net;

import android.graphics.PointF;


/**
 * It is the full motion event package.
 * @author CHEN JIAN WEN
 *
 */
public class FullMotionEventPackage extends DataPackage {

    /**
     * It is the motion event source.
     */
    public int mSource;

    /**
     * It is the motion event action.
     */
    public int mAction;

    /**
     * It is the flag of motion event.
     */
    public int mFlags;

    /**
     * It is the point number.
     */
    public int mPointNumber;

    /**
     * It is the point set.
     */
    public PointF[] mPoints;

    /**
     * It is the point id set.
     */
    public int[] mIds;

    /**
     * It constructs the motion event package.
     */
    public FullMotionEventPackage() {
        mType = PKG_TYPE_FULL_MOTION_EVENT;
    }

    /**
     * It sets the motion event of the package.
     * @param source It is the motion event source.
     * @param action It is the motion event action.
     * @param x It is the point value in x axis.
     * @param x It is the point value in y axis.
     */
    public void setMotionEvent(int source, int action, float x, float y) {
        mSource = source;
        mAction = action;
        mFlags = 0;
        mPointNumber = 1;
        mPoints = new PointF[1];
        mIds = new int[1];
        mPoints[0]= new PointF(x, y);
        mIds[0] = 0;
    }

    public void setMotionEvent(int source, int action, int flags, PointF[] points, int[] ids) {
        if ((points == null) || (ids == null)) return;

        final int length = Math.min(points.length, ids.length);
        mSource = source;
        mAction = action;
        mFlags = flags;
        mPointNumber = length;
        mPoints = new PointF[length];
        mIds = new int[length];
        for (int i = 0; i < points.length; i++) {
            mPoints[i] = new PointF(points[i].x, points[i].y);
            mIds[i] = ids[i];
        }
    }

    /**
     * It converts the data to buffer.
     * @return It returns the byte array buffer with data
     */
    private byte[] dataToBuffer() {
        final int size = 16 + 12 * mPointNumber;

        final byte[] data = new byte[size];

        int i = 0;
        System.arraycopy(Utils.intToByteArray(mSource), 0, data, i, 4);
        i += 4;

        System.arraycopy(Utils.intToByteArray(mAction), 0, data, i, 4);
        i += 4;

        System.arraycopy(Utils.intToByteArray(mFlags), 0, data, i, 4);
        i += 4;

        System.arraycopy(Utils.intToByteArray(mPointNumber), 0, data, i, 4);
        i += 4;

        for (int j = 0; j < mPointNumber; j++) {
            System.arraycopy(Utils.floatToByteArray(mPoints[j].x), 0, data, i, 4);
            i += 4;

            System.arraycopy(Utils.floatToByteArray(mPoints[j].y), 0, data, i, 4);
            i += 4;

            System.arraycopy(Utils.intToByteArray(mIds[j]), 0, data, i, 4);
            i += 4;
        }
        return data;
    }

    /**
     * It converts the buffer to value.
     * @param buffer It is the buffer to convert.
     */
    private void bufferToData(byte[] buffer) {
        byte buf[] = new byte[4];
        int i = 0;

        System.arraycopy(buffer, i, buf, 0, 4);
        mSource = Utils.byteArrayToInt(buf);
        i += 4;

        System.arraycopy(buffer, i, buf, 0, 4);
        mAction = Utils.byteArrayToInt(buf);
        i += 4;

        System.arraycopy(buffer, i, buf, 0, 4);
        mFlags = Utils.byteArrayToInt(buf);
        i += 4;

        System.arraycopy(buffer, i, buf, 0, 4);
        mPointNumber = Utils.byteArrayToInt(buf);
        i += 4;

        mPoints = new PointF[mPointNumber];
        mIds = new int[mPointNumber];
        for (int j = 0; j < mPointNumber; j++) {
            System.arraycopy(buffer, i, buf, 0, 4);
            final float x = Utils.byteArrayToFloat(buf);
            i += 4;
    
            System.arraycopy(buffer, i, buf, 0, 4);
            final float y = Utils.byteArrayToFloat(buf);
            i += 4;

            mPoints[j] = new PointF(x, y);

            System.arraycopy(buffer, i, buf, 0, 4);
            mIds[j] = Utils.byteArrayToInt(buf);
            i += 4;
        }
    }

    /**
     * It reads the data from source.
     * <BR>
     * <B> Note </B> <BR>
     * You must make an exception protection when you use this method.
     * @param in It is data source.
     */
    public void read(byte[] in) {
        final byte blen[] = new byte[4];
        int i = DataPackage.PKG_HEADER_SIZE;
        System.arraycopy(in, i, blen, 0, 4);
        i += 4;
        final int len = Utils.byteArrayToInt(blen);
        if (len > 0) {
            final byte b[] = new byte[len];
            System.arraycopy(in, i, b, 0, len);
            bufferToData(b);
        }
    }

    /**
     * It write the data to target buffer.
     * <BR>
     * <B> Note </B> <BR>
     * You must make an exception protection when you use this method.
     * @return It returns the data buffer to write.
     */
    public byte[] write() {
        final byte buffer[] = dataToBuffer();
        final int len = buffer.length;
        final byte blen[] = Utils.intToByteArray(len);
        int i = DataPackage.PKG_HEADER_SIZE;
        final byte out[] = new byte[len + 4 + i];
        System.arraycopy(blen, 0, out, i, 4);
        i += 4;
        System.arraycopy(buffer, 0, out, i, len);
        return out;
    }
}