package com.oozic.net;

import android.util.Log;


/**
 * It is the base data to transmit data on net. All the data that inherit from this class should
 * follow the following rule,<BR><BR>
 * Id [4] <BR>
 * Key [4] <BR>
 * Byte Array Size [4] <BR>
 * Item Number [4] <BR>
 * Item1 Array Size [4] <BR>
 * Item2 Array Size [4] <BR>
 * ... <BR>
 * ItemN Array Size [4] <BR>
 * Item1 Array Data [?] <BR>
 * Item2 Array Data [?] <BR>
 * ... <BR>
 * ItemN Array Data [?] <BR>
 * <B> Note, All protocol Id like 0x0051Cxxx and 0x0021Cxxx are occupied by net comonent, so DO NOT
 * use!!
 * @author CHEN JIAN WEN
 */
public abstract class NetData {
    /**
     * It is the file transceive request protocol id.<BR>
     * This ID is occupied by net component file direct transceive(FDT) module.
     */
    public static final int ID_FILE_TRANSCEIVE_REQ = 0x0051CF70;

    /**
     * It is the file transceive acknowledge protocol id. If this data is
     * received, it means the file is accepted by target net component.<BR>
     * This ID is occupied by net component file direct transceive(FDT) module.
     */
    public static final int ID_FILE_TRANSCEIVE_ACK = 0x0051CF71;

    /**
     * It is the file data transceive protocol id.<BR>
     * This ID is occupied by net component file direct transceive(FDT) module.
     */
    public static final int ID_FILE_TRANSCEIVE_DATA = 0x0051CF72;

    /**
     * It is the file transceive error protocol id. If this data is
     * received, an error occurs on the target net component.<BR>
     * This ID is occupied by net component file direct transceive(FDT) module.
     */
    public static final int ID_FILE_TRANSCEIVE_ERROR = 0x0051CF73;

    /**
     * It is the class tag.
     */
    private static final String TAG = "NetData";

    /**
     * It is the debug switch.
     */
    static final boolean DEBUG = NetManager.DEBUG;

    /**
     * It is the protocol id offset in the buffer.
     */
    protected static final int ID_OFFSET = 0;

    /**
     * It is the key offset in the buffer.
     */
    protected static final int KEY_OFFSET = 4;

    /**
     * It is the length offset in the buffer.
     */
    protected static final int LENGTH_OFFSET = 8;

    /**
     * It is the item number offset in the buffer.
     */
    protected static final int ITEM_NUMBER_OFFSET = 12;

    /**
     * It is the item size offset in the buffer.
     */
    protected static final int ITEM_SIZE_OFFSET = 16;

    /**
     * It is the fixed head size of net data.
     */
    public static final int FIXED_HEAD_SIZE = ITEM_SIZE_OFFSET;

    /**
     * It is the string coding.
     */
    protected static final String STRING_CODING = "utf-8";

    /**
     * It is the NetDataEmpty type.
     */
    public static final int TYPE_EMPTY = 0;

    /**
     * It is the NetDataContainer type.
     */
    public static final int TYPE_CONTAINER = 1;

    /**
     * It is the NetDataFile type.
     */
    public static final int TYPE_FILE = 2;

    /**
     * It is the NetDataFileHead type.
     */
    public static final int TYPE_FILE_HEAD = 3;

    /**
     * It is the NetDataInteger type.
     */
    public static final int TYPE_INT = 4;

    /**
     * It is the NetDataIntegers type.
     */
    public static final int TYPE_INTS = 5;

    /**
     * It is the NetDataString type.
     */
    public static final int TYPE_STRING = 6;

    /**
     * It is the NetDataStrings type.
     */
    public static final int TYPE_STRINGS = 7;

    /**
     * It is the NetDataStringsBitmaps type.
     */
    public static final int TYPE_STRINGS_BITMAPS = 8;

    /**
     * It is the NetDataIntegersStrings type.
     */
    public static final int TYPE_INTS_STRINGS = 9;

    /**
     * It logs information.
     * @param text It is the text to log.
     */
    final void log_i(String text) {
        if (DEBUG) Log.i(TAG, text);
    }

    /**
     * It logs warnings.
     * @param text It is the text to log.
     */
    final void log_w(String text) {
        if (DEBUG) Log.w(TAG, text);
    }

    /**
     * It is the key of the the data. A key is used when the package action is not a request.
     * A request brings a key to target, and the response of to the client must contains this key.  
     */
    protected int mKey;

    /**
     * It is the protocol id.
     */
    protected int mID;

    /**
     * It converts the net data to byte array.
     * <BR>
     * <B><FONT color=RED>PLEASE FORM YOUR DATA WITH THE RULE OF NET DATA.</FONT></B>
     * @return It returns the net data in byte array form.
     */
    abstract public byte[] toBytes();

    /**
     * It converts the net data from byte array form.
     * @param bytes It is the byte array to convert.
     * @return It returns true if the convertion is successful. And it returns false otherwise.<BR>
     * <B><FONT color=RED>THE MEMBERS MAY DIRTY WHEN THE METHOD RETURN FALSE.</FONT></B>
     */
    abstract public boolean fromBytes(byte[] bytes);

    /**
     * It checks the byte array if the data is valid. Call it in fromBytes() method to read out
     * the byte array size of each item buffer.
     * <B>Note</B>, in order to make the check operation faster, we only check the size of the
     * byte array.
     * @param bytes
     * @return It returns an size array of items if the bytes buffer is ok. Otherwise, it returns
     * null.
     * <BR>
     * With the returned size array, we can also get the first data index with following formula,
     * <BR>
     * <I><FONT color=BLUE>index = 16 + array.length * 4.</FONT></I> 
     */
    protected final int[] checkBytes(byte[] bytes) {
        if (bytes.length < 16) return null;

        byte tmp[] = new byte[4];
        final int length = getLength(bytes);

        if (length != bytes.length) {
            log_w("length error!" + bytes.length + ":" + length);
            return null;
        }

        final int itemNumber = getItemNumber(bytes);

        if (itemNumber < 0) {
            return null;
        }

        final int ret[] = new int[itemNumber];

        int i = ITEM_SIZE_OFFSET;
        int l = i;
        for (int x = 0; x < itemNumber; x ++) {
            try {
                System.arraycopy(bytes, i, tmp, 0, 4);
                ret[x] = Utils.byteArrayToInt(tmp);
                log_i("size[" + x +"]:" + ret[x]);
            } catch (IndexOutOfBoundsException e) {
                return null;
            }

            l += ret[x] + 4;
            i += 4;
        }
        
        if (l != length) {
            log_w("data length error!" + l + ":" + length);
            return null;
        }

        return ret;
    }

    /**
     * It sets the key value.
     * @param key It is the key value to set.
     */
    public final void setKey(int key) {
        mKey = key;
    }

    /**
     * It gets the key from current object.
     * @return It returns a key from current object.
     */
    public int getKey() {
        return mKey;
    }

    /**
     * It gets the id object from the byte array.
     * @return It returns the id.
     */
    public static int getId(byte[] bytes) {
        if (bytes.length < 16) return 0;
        byte tmp[] = new byte[4];
        try {
            System.arraycopy(bytes, ID_OFFSET, tmp, 0, 4);
        } catch (IndexOutOfBoundsException e) {
            return 0;
        }
        return Utils.byteArrayToInt(tmp);
    }

    /**
     * It gets the key object from the byte array.
     * @param bytes It is the net data buffer.
     */
    public static int getKey(byte[] bytes) {
        if (bytes.length < 16) return 0;
        byte tmp[] = new byte[4];
        try {
            System.arraycopy(bytes, KEY_OFFSET, tmp, 0, 4);
        } catch (IndexOutOfBoundsException e) {
            return 0;
        }

        return Utils.byteArrayToInt(tmp);
    }

    /**
     * It gets the length object from the byte array.
     * @param bytes It is the net data buffer.
     */
    public static int getLength(byte[] bytes) {
        if (bytes.length < 16) return 0;
        byte tmp[] = new byte[4];
        try {
            System.arraycopy(bytes, LENGTH_OFFSET, tmp, 0, 4);
        } catch (IndexOutOfBoundsException e) {
            return 0;
        }

        return Utils.byteArrayToInt(tmp);
    }

    /**
     * It gets the item number object from the byte array.
     * @param bytes It is the net data buffer.
     */
    public static int getItemNumber(byte[] bytes) {
        if (bytes.length < 16) return 0;
        byte tmp[] = new byte[4];
        try {
            System.arraycopy(bytes, ITEM_NUMBER_OFFSET, tmp, 0, 4);
        } catch (IndexOutOfBoundsException e) {
            return 0;
        }

        return Utils.byteArrayToInt(tmp);
    }

    /**
     * It prepares a head for net data.
     * @param buffer It is the buffer to write in.
     * @param length It is the data length.
     * @param itemNumber It is the item number.
     */
    protected void writeHead(byte[] buffer, int length, int itemNumber) {
        int i = 0;
        // Set ID.
        System.arraycopy(Utils.intToByteArray(mID), 0, buffer, i, 4);
        i += 4;

        // Set Key
        System.arraycopy(Utils.intToByteArray(mKey), 0, buffer, i, 4);
        i += 4;

        // Set data length
        System.arraycopy(Utils.intToByteArray(length), 0, buffer, i, 4);
        i += 4;

        // Set data field number.
        System.arraycopy(Utils.intToByteArray(itemNumber), 0, buffer, i, 4);
        //i += 4;
        return;
    }

    /**
     * It gets the net data type.
     * @return It returns the net data type.
     */
    protected int getNetDataType() {
        if (this instanceof NetDataEmpty) {
            return TYPE_EMPTY;
        } else if (this instanceof NetDataContainer) {
            return TYPE_CONTAINER;
        } else if (this instanceof NetDataFile) {
            return TYPE_FILE;
        } else if (this instanceof NetDataFileHead) {
            return TYPE_FILE_HEAD;
        } else if (this instanceof NetDataInteger) {
            return TYPE_INT;
        } else if (this instanceof NetDataIntegers) {
            return TYPE_INTS;
        } else if (this instanceof NetDataString) {
            return TYPE_STRING;
        } else if (this instanceof NetDataStrings) {
            return TYPE_STRINGS;
        } else if (this instanceof NetDataStringsBitmaps) {
            return TYPE_STRINGS_BITMAPS;
        } else if (this instanceof NetDataIntegersStrings) {
            return TYPE_INTS_STRINGS;
        }
        return -1;
    }

    /**
     * It gets the net data.
     * @param type It is the net data type.
     * @param buffer It is the buffer to initialize the net data.
     */
    protected NetData getNetData(int type, byte[] buffer) {
        NetData data = null;
        final int id = NetData.getId(buffer);
        switch (type) {
        case TYPE_EMPTY:
            data = new NetDataEmpty(id);
            break;
        case TYPE_CONTAINER:
            data = new NetDataContainer(id);
            break;
        case TYPE_FILE:
            data = new NetDataFile(id);
            break;
        case TYPE_FILE_HEAD:
            data = new NetDataFileHead(id);
            break;
        case TYPE_INT:
            data = new NetDataInteger(id);
            break;
        case TYPE_INTS:
            data = new NetDataIntegers(id);
            break;
        case TYPE_STRING:
            data = new NetDataString(id);
            break;
        case TYPE_STRINGS:
            data = new NetDataStrings(id);
            break;
        case TYPE_STRINGS_BITMAPS:
            data = new NetDataStringsBitmaps(id);
            break;
        case TYPE_INTS_STRINGS:
            data = new NetDataIntegersStrings(id);
            break;
        default:
            android.util.Log.w(TAG, "Unsupported data type!");
            break;
        }

        if (data != null) {
            if (!data.fromBytes(buffer)) {
                data = null;
            }
        }
        return data;
    }
	
}
