package com.oozic.net;

import java.util.ArrayList;

/**
 * File transfer task tells the file sender how and what to send to the target.
 * @author CHEN JIAN WEN
 */
public class NetDataFileTransferTask extends NetData {

    /**
     * It is the tasks for file sender to send.
     */
    protected ArrayList<Task> mTasks;

    /**
     * It is the block size for every file data block.
     */
    protected int mBlockSize;


    /**
     * It is the key to send data;
     */
    protected int mDataKey;

    /**
     * It constructs the file transfer task net data without initializing the members.
     * @param id It is the protocol id. 
     */
    public NetDataFileTransferTask(int id) {
        mID = id;
    }

    /**
     * It constructs the file transfer task net data.
     * @param id It is the protocol id.
     * @param blockSize It is the file data block size.
     * @param blockIndexes It is the block indexes.
     */
    public NetDataFileTransferTask(int id, int blockSize, int dataKey, int ... blockIndexes) {
        mID = id;
        mTasks = new ArrayList<Task>();
        for (int p : blockIndexes) {
            final Task task = new Task(p);
            mTasks.add(task);
        }
        mDataKey = dataKey;
        mBlockSize = blockSize;
    }

    @Override
    public byte[] toBytes() {
        final int length = FIXED_HEAD_SIZE + 12 + mTasks.size() * 4 + 8; 

        byte mainBuffer[] = new byte[length];
        int i = ITEM_SIZE_OFFSET;

        writeHead(mainBuffer, length, 3);

        // +<<------------------------------------------------------------------------------------+
        // | Set the size of task data.                                                           |
        // +--------------------------------------------------------------------------------------+
        System.arraycopy(Utils.intToByteArray(mTasks.size() * 4), 0, mainBuffer, i, 4);
        i += 4;
        // +<<------------------------------------------------------------------------------------+
        // | Set the size of file data block size, it's always 4.                                 |
        // +--------------------------------------------------------------------------------------+
        System.arraycopy(Utils.intToByteArray(4), 0, mainBuffer, i, 4);
        i += 4;
        // +<<------------------------------------------------------------------------------------+
        // | Set the size of data key, it's always 4.                                             |
        // +--------------------------------------------------------------------------------------+
        System.arraycopy(Utils.intToByteArray(4), 0, mainBuffer, i, 4);
        i += 4;

        // +<<------------------------------------------------------------------------------------+
        // | Set the data of task.                                                                |
        // +--------------------------------------------------------------------------------------+
        for (Task p : mTasks) {
            System.arraycopy(Utils.intToByteArray(p.mBlockIndex), 0, mainBuffer, i, 4);
            i += 4;
        }
        // +<<------------------------------------------------------------------------------------+
        // | Set the data of file data block size.                                                |
        // +--------------------------------------------------------------------------------------+
        System.arraycopy(Utils.intToByteArray(mBlockSize), 0, mainBuffer, i, 4);
        i += 4;
        // +<<------------------------------------------------------------------------------------+
        // | Set the data key.                                                                    |
        // +--------------------------------------------------------------------------------------+
        System.arraycopy(Utils.intToByteArray(mDataKey), 0, mainBuffer, i, 4);
        //i += 4;

        return mainBuffer;
    }

    @Override
    public boolean fromBytes(byte[] bytes) {
        final int lens[] = checkBytes(bytes);

        if (lens == null) return false;
        byte buf[];
        int i = FIXED_HEAD_SIZE + lens.length * 4;
        int j = 0;

        final ArrayList<Task> tasks = new ArrayList<Task>();
        try {
            // +<<------------------------------------------------------------------------------------+
            // | Get the file path.                                                                   |
            // +--------------------------------------------------------------------------------------+
            buf = new byte[lens[j]];
            System.arraycopy(bytes, i, buf, 0, lens[j]);
            final int n = lens[j] / 4;
            final byte [] b4 = new byte[4];
            for (int k = 0; k < n ; k++) {
                System.arraycopy(buf, k * 4, b4, 0, 4);
                final Task task = new Task(Utils.byteArrayToInt(b4));
                tasks.add(task);
            }
            i += lens[j];
            j ++;

            // +<<------------------------------------------------------------------------------------+
            // | Get the file data block size.                                                        |
            // +--------------------------------------------------------------------------------------+
            buf = new byte[lens[j]];
            System.arraycopy(bytes, i, buf, 0, lens[j]);
            mBlockSize = Utils.byteArrayToInt(buf);
            i += lens[j];
            j ++;

            // +<<------------------------------------------------------------------------------------+
            // | Get the data key.                                                                    |
            // +--------------------------------------------------------------------------------------+
            buf = new byte[lens[j]];
            System.arraycopy(bytes, i, buf, 0, lens[j]);
            mDataKey = Utils.byteArrayToInt(buf);
            //i += lens[j];
            //j ++;
        } catch (IndexOutOfBoundsException e) {
            e.printStackTrace();
            return false;
        }
        mTasks = tasks;
        return true;
    }

    /**
     * It gets the file block size.
     * @return It returns the file block size.
     */
    public int getFileBlockSize() {
        return mBlockSize;
    }

    /**
     * It gets the file block indexes to send.
     * @return It returns the file block indexes.
     */
    public int[] getFileBlockIndexes() {
        final int[] indexes = new int[mTasks.size()];
        int i = 0;
        for (Task p : mTasks) {
            indexes[i] = p.mBlockIndex;
            i++;
        }
        return indexes;
    }

    /**
     * It gets the data key.
     * @return It returns the data key.
     */
    public int getDataKey() {
        return mDataKey;
    }

    /**
     * It is the file transfer task.
     * @author CHEN JIAN WEN
     *
     */
    public class Task {
        /**
         * It is the file data block index.
         */
        int mBlockIndex;

        /**
         * It constructs the task.
         * @param blockIndex It is the block index to set.
         */
        public Task(int blockIndex) {
            mBlockIndex = blockIndex;
        }
    }
}
