package com.oozic.net;

import android.util.Log;

import com.oozic.time.TimeTrigger;

/**
 * It is the net package receiver. A net package receiver contains the processes for a special net
 * package.
 * @author CHEN JIAN WEN.
 */
class NetDataReceiver {
    /**
     * It is the log tag of this class.
     */
    private static final String TAG = "NetDataReceiver";

    /**
     * It is the debug switch.
     */
    static final boolean DEBUG = NetManager.DEBUG;

    /**
     * It logs information.
     * @param text It is the text to log.
     */
    final void log_i(String text) {
        if (DEBUG) Log.i(TAG, text);
    }

    /**
     * It logs warnings.
     * @param text It is the text to log.
     */
    final void log_w(String text) {
        if (DEBUG) Log.w(TAG, text);
    }

    /**
     * It logs errors.
     * @param text It is the text to log.
     */
    final void log_e(String text) {
        if (DEBUG) Log.e(TAG, text);
    }

    /**
     * It is the package action to trigger.
     */
    protected int mAction;

    /**
     * It is the id to trigger.
     */
    protected int mId;

    /**
     * It is the key to trigger.
     */
    protected int mKey;

    /**
     * It is the process that runs when all trigger condition is matched.
     */
    protected NetDataReceiverListener mProcess;

    /**
     * It is the time out trigger.
     */
    protected TimeTrigger mTimeout;

    /**
     * It is the trigger manager.
     */
    protected NetDataReceiverManager mManager;

    /**
     * It constructs the process trigger.
     * <BR>
     * <B> DO NOT USE THIS CONSTRUCTOR IN YOUR PROGRAME!</B>
     * Just use the createReceiver() in manager.
     * @param action It is the action to trigger.
     * @param id It is the id to trigger.
     * @param key It is the key to trigger.
     * @param process It is the process that runs when all trigger condition is matched.
     * @param mananger It is the trigger mananger.
     * @param infinit It makes a decision on if the trigger need a time out machanism.
     */
    NetDataReceiver(int action, int id, int key,
            NetDataReceiverListener process,
            NetDataReceiverManager manager,
            boolean infinit) {
        mAction = action;
        mId = id;
        mKey = key;
        mProcess = process;
        mManager = manager;
        if (!infinit) {
            mTimeout = new TimeTrigger();
            touch();
        }
    }

    /**
     * It makes process run if all trigger condition is matched.
     * <BR>
     * <B> DO NOT USE THIS METHOD IN YOUR PROGRAME!</B>
     * It will be called by the manager.
     * @param action It is the package action to compare.
     * @param id It is the id to compare.
     * @param key It is the key to compare.
     * @param pack It is the data package to process.
     * @param nui It is the data package source.
     * @return It returns true if all the condition is matched and returns false otherwise.
     */
    boolean runIfMatch(
            int action, int id, int key,
            ByteBufferPackage pack, NetUser nui) {
        if ((action == mAction) &&(id == mId) &&
                ((action == DataPackage.ACTION_REQUEST) || (key == mKey))) {
            log_i("Match Action:" + mAction + " ["
                    + String.format("0x%08x",mId) + ":" + mKey + "]");
            if (mProcess != null) {
                touch();
                mProcess.run(key, pack, nui);
            }
            return true;
        }
//        else {
//            log_e("Unmatch Action:" + action + " ["
//                    + String.format("0x%08x",id) + ":" + key + "]");
//        }
        return false;
    }

    /**
     * It checks if all trigger condition is matched.
     * <BR>
     * <B> DO NOT USE THIS METHOD IN YOUR PROGRAME!</B>
     * It will be called by the manager.
     * @param action It is the package action to compare.
     * @param id It is the id to compare.
     * @param key It is the key to compare.
     * @return It returns true if all the condition is matched and returns false otherwise.
     */
    boolean isMatch(int action, int id, int key) {
        if ((action == mAction) &&(id == mId) &&
                ((action == DataPackage.ACTION_REQUEST) || (key == mKey))) {
            return true;
        }
        return false;
    }

    /**
     * It touchs the timeout trigger to reset it.
     * <BR>
     * <B> DO NOT USE THIS METHOD IN YOUR PROGRAME!</B>
     * It will be called by the trigger manager.
     */
    void touch() {
        if (mTimeout == null) return;
        mTimeout.clear();
        mTimeout.start(10000, new Runnable() {
            @Override
            public void run() {
                if (mProcess != null) mProcess.onTimeOut(mAction, mId, mKey);
                mManager.kill(NetDataReceiver.this);
            }
        });
    }

    /**
     * It destroy the receiver. It will be called by the manager.
     */
    void destroy() {
        if (mTimeout != null) {
            mTimeout.clear();
            mTimeout = null;
        }
        mAction = 0;
        mId = 0;
        mKey = 0;
        mProcess = null;
        mManager = null;
    }
}
