package com.oozic.net;

import java.util.ArrayList;

/**
 * It is the net data receiver manager. A net data receiver manager maintains a mechanism
 * which help user manage the net data receiver.
 * @author CHEN JIAN WEN.
 */
public class NetDataReceiverManager {

    /**
     * It is the receiver set.
     */
    protected ArrayList<NetDataReceiver> mReceivers = null;

    /**
     * It is the synchronize object.
     */
    protected Object mSync = new Object();

    /**
     * It is the thread to process net data. 
     */
    NetDataThread mThread;

    /**
     * It constructs the trigger mananger.
     */
    NetDataReceiverManager() {
        mReceivers = new ArrayList<NetDataReceiver>();
        mThread = new NetDataThread(this);
        mThread.start();

        // Make sure the thread started.
        try {
            Thread.sleep(33);
        } catch (InterruptedException e) {
            e.printStackTrace();
        }
    }

    /**
     * It creates a trigger add it to manager.
     * @param action It is the action to trigger.
     * @param id It is the id to trigger.
     * @param key It is the key to trigger.
     * @param process It is the process that runs when all trigger condition is matched.
     * @param infinit It makes a decision on if the trigger need a time out machanism.
     */
    public void createReceiver(int action, int id, int key, NetDataReceiverListener process, boolean infinit) {
        final NetDataReceiver receiver =
            new NetDataReceiver(action, id, key, process, this, infinit);
        synchronized(mSync) {
            mReceivers.add(receiver);
            NetManager.log_i(String.format(
                    "-- Create Receiver|Action: 0x%08x|Id: 0x%08x|Key: 0x%08x", action, id, key));
            NetManager.log_i(
                    "-- Receiver Number: " + mReceivers.size());
        }
    }

    /**
     * It creates a key.
     * @param action It is the package action.
     * @param id It is the id.
     */
    public int createKey(int action, int id) {
        boolean repeated = false;
        int key;
        do {
            repeated = false;
            if (Math.random() > 0.5) {
                key = (int)(Integer.MAX_VALUE * Math.random());
            } else {
                key = (int)(Integer.MIN_VALUE * Math.random());
            }
            synchronized(mSync) {
                for (NetDataReceiver p : mReceivers) {
                    if (p.isMatch(action, id, key)) {
                        repeated = true;
                        break;
                    }
                }
            }
        }while(repeated);
        return key;
    }

    /**
     * It is used to kill and remove the trigger from mananger.
     * @param trigger It is the receiver to remove.
     */
    void kill(NetDataReceiver receiver) {
        synchronized (mSync) {
            if (receiver != null) {
                mReceivers.remove(receiver);
                receiver.destroy();
                NetManager.log_w(String.format(
                        "-- Kill Receiver|Action: 0x%08x|Id: 0x%08x|Key: 0x%08x",
                        receiver.mAction,
                        receiver.mId,
                        receiver.mKey));
                NetManager.log_w("-- Receiver Number: " + mReceivers.size());
            }
        }
    }

    /**
     * It kill receiver if all trigger condition is matched.
     * @param action It is the package action to compare.
     * @param id It is the id to compare.
     * @param key It is the key to compare.
     */
    public void kill(int action, int id, int key) {
        synchronized(mSync) {
            for (NetDataReceiver p : mReceivers) {
                if (p.isMatch(action, id, key)) {
                    kill(p);
                    return;
                }
            }
        }
    }

    /**
     * It request to process the data.
     * @param pack It is the package to process.
     * @param nu It is the user who sends the package.
     */
    void requestToProcess(ByteBufferPackage pack, NetUser nu) {
        mThread.processData(pack, nu);
    }

    /**
     * It makes process run if all trigger condition is matched.
     * @param pack It is the data package to process.
     * @param nui It is the data package source.
     */
    void runIfMatch(ByteBufferPackage pack, NetUser nui) {
        final int action = pack.getAction();
        final int id = NetData.getId(pack.getByteBuffer());
        final int key = NetData.getKey(pack.getByteBuffer());
        synchronized(mSync) {
            NetManager.log_w(String.format("-- Check package|Action: %x|ID: %x|Key: %x", action, id, key));
            NetManager.log_w("-- From: "  + nui.mName + "|" + nui.getIpAddress() + ":" +nui.getSocketPort());
            for (NetDataReceiver p : mReceivers) {
                if (p.runIfMatch(action, id, key, pack, nui)) {
                    NetManager.log_i(String.format("-- Process package|Action: %x|ID: %x|Key: %x", action, id, key));
                    NetManager.log_i("-- From: "  + nui.mName + "|" + nui.getIpAddress() + ":" +nui.getSocketPort());
                    break;
                }
            }
        }
    }

    /**
     * Touches the receiver to reset the timeout trigger.
     * @param action It is the package action to compare.
     * @param id It is the id to compare.
     * @param key It is the key to compare.
     */
    public void touch(int action, int id, int key) {
        synchronized(mSync) {
            for (NetDataReceiver p : mReceivers) {
                if (p.isMatch(action, id, key)) {
                    p.touch();
                    return;
                }
            }
        }
    }

    /**
     * It destroy the net data receiver manager.
     */
    void destroy() {
        mThread.quit();
        synchronized(mSync) {
            for (NetDataReceiver p : mReceivers) {
                p.destroy();
            }
            mReceivers.clear();
        }
    }
}
