package com.oozic.net;

import java.io.ByteArrayOutputStream;
import java.io.UnsupportedEncodingException;
import java.util.ArrayList;

import com.oozic.apps.Util;
import com.oozic.bitmap.BitmapSpyManager;
import com.oozic.net.NetData;
import com.oozic.net.Utils;

import android.graphics.Bitmap;
import android.graphics.Bitmap.CompressFormat;
import android.graphics.Bitmap.Config;
import android.graphics.BitmapFactory;

/**
 * It is the net data with strings and bitmaps.
 * @author CHEN JIAN WEN
 *
 */
public class NetDataStringsBitmaps extends NetData {
    /**
     * It is the field number in this class.
     */
    private int mItemNumber;

    /**
     * It is the strings.
     */
    private String mStrings[];

    /**
     * It is the bitmaps.
     */
    private Bitmap mBitmaps[];

    /**
     * Since this class includes a bitmap field, we need a bitmap spy to manage it.
     */
    protected BitmapSpyManager mSpy;

    /**
     * It constructs the empty net data of strings and bitmaps type.
     * @param id It is the protocol id.
     */
    protected NetDataStringsBitmaps(int id) {
        this(id, null, null, null);
    }

    /**
     * It constructs an empty application information.
     * @param id It is the protocol id.
     * @param spy It is the bitmap spy manager.
     */
    protected NetDataStringsBitmaps(int id, BitmapSpyManager spy) {
        this(id, null, null, spy);
    }

    /**
     * It constructs the application information with a resolve information.
     * @param id It is the protocol id.
     * @param strings It is the string set.
     * @param bitmaps It is the bitmap set.
     * @param spy It is the bitmap spy manager.
     */
    protected NetDataStringsBitmaps(int id, String[] strings, Bitmap[] bitmaps, BitmapSpyManager spy) {
        if (strings == null) mStrings = new String[0];
        else mStrings = strings;

        if (bitmaps == null) mBitmaps = new Bitmap[0];
        else mBitmaps = bitmaps;

        mSpy = spy;

        mItemNumber = mStrings.length + mBitmaps.length + 2;
        
        mID = id;
    }

    @Override
    public byte[] toBytes() {
        final ArrayList<byte[]> mBufferArray = new ArrayList<byte[]>();

        int length = 16;

        mBufferArray.add(Util.intToByteArray(mStrings.length));
        length += 8;

        for(int i = 0; i < mStrings.length; i++) {
            byte buffer[];
            try {
                buffer = mStrings[i].getBytes(STRING_CODING);
            } catch (UnsupportedEncodingException e) {
                buffer = mStrings[i].getBytes();
            } catch (NullPointerException e) {
                buffer = new byte[0];
            }
            mBufferArray.add(buffer);
            length += 4 + buffer.length;
        }

        mBufferArray.add(Util.intToByteArray(mBitmaps.length));
        length += 8;

        final ByteArrayOutputStream os = new ByteArrayOutputStream();
        for(int i = 0; i < mBitmaps.length; i++) {
            byte buffer[];
            try {
                mBitmaps[i].compress(CompressFormat.PNG, 100, os);
                buffer = os.toByteArray();
            } catch(NullPointerException e) {
                buffer = new byte[0];
            }
            mBufferArray.add(buffer);
            length += 4 + buffer.length;
        };

        byte ret[] = new byte[length];
        int i = 0;

        // Set ID.
        System.arraycopy(Utils.intToByteArray(mID), 0, ret, i, 4);
        i += 4;

        // Set Key
        System.arraycopy(Utils.intToByteArray(mKey), 0, ret, i, 4);
        i += 4;

        // Set the data length.
        System.arraycopy(Utils.intToByteArray(length), 0, ret, i, 4);
        i += 4;

        // Set the item number.
        System.arraycopy(Utils.intToByteArray(mItemNumber), 0, ret, i, 4);
        i += 4;

        // Set the data size filed
        for (byte[] p : mBufferArray) {
            System.arraycopy(Utils.intToByteArray(p.length), 0, ret, i, 4);
            i += 4;
        }

        // Set the data field
        for (byte[] p : mBufferArray) {
            System.arraycopy(p, 0, ret, i, p.length);
            i += p.length;
        }
        return ret;
    }

    @Override
    public boolean fromBytes(byte[] bytes) {
        final int lens[] = checkBytes(bytes);

        if (lens == null) return false;
        byte buf[];
        int i = 16 + lens.length * 4;
        int j = 0, j1, n;

        // Get number of string.
        buf = new byte[lens[j]];
        System.arraycopy(bytes, i, buf, 0, lens[j]);
        n = Util.byteArrayToInt(buf);
        mStrings = new String[n];
        i += lens[j];
        j ++;
        n += j;

        // Get strings.
        j1 = j;
        for (; j < n ; j++) {
            if (lens[j] == 0) {
                mStrings[j - j1] = null;
                continue;
            }
            buf = new byte[lens[j]];
            System.arraycopy(bytes, i, buf, 0, lens[j]);
            try {
                mStrings[j - j1] = new String(buf, STRING_CODING);
            } catch (UnsupportedEncodingException e) {
                mStrings[j - j1] = new String(buf);
            }
            i += lens[j];
        }

        // Get number of bitmap.
        buf = new byte[lens[j]];
        System.arraycopy(bytes, i, buf, 0, lens[j]);
        n = Util.byteArrayToInt(buf);
        mBitmaps = new Bitmap[n];
        i += lens[j];
        j ++;
        n += j;

        // Get bitmaps.
        j1 = j;
        for (; j < n ; j++) {
            if (lens[j] == 0) {
                mBitmaps[j - j1] = null;
                continue;
            }
            buf = new byte[lens[j]];
            System.arraycopy(bytes, i, buf, 0, lens[j]);
            if (mSpy != null) {
                Bitmap bmp = null;
                bmp = mSpy.decodeByteArray(buf, 0, buf.length, Config.ARGB_8888, false, true);
                if (bmp == null) {
                    bmp = mSpy.decodeByteArray(buf, 0, buf.length, Config.ARGB_8888, true, true);
                }
                mBitmaps[j - j1] = bmp;
            } else {
                try {
                    mBitmaps[j - j1] = BitmapFactory.decodeByteArray(buf, 0, buf.length);
                } catch (OutOfMemoryError e) {
                    mBitmaps[j - j1] = null;
                }
            }
            i += lens[j];
        }

        return true;
    }

    /**
     * It gets the specified string.
     * @param index It is the index of string.
     * @return It gets the specified string.
     */
    protected String getString(int index) {
        return mStrings[index];
    }

    /**
     * It gets the specified bitmap.
     * @param index It is the index of the bitmap.
     * @return It returns the specified bitmap.
     */
    protected Bitmap getBitmap(int index) {
        return mBitmaps[index];
    }
}
