package com.oozic.net;

import android.content.Context;
import android.net.wifi.WifiManager;
import android.os.Build;
import android.os.Handler;
import android.util.Log;

import com.oosic.net.connectivity.ConnectivityCenter;
import com.oozic.apps.Util;
import com.oozic.library.http.MediaServer;
import com.oozic.library.sdsp.SdspManager;

/**
 * It is the share manager controller.
 * @author CHEN JIAN WEN
 */
public class NetManager {
    /**
     * It is the tag of the class.
     */
    private static final String TAG = "NetManager";

    /**
     * It is the debug switch.
     */
    static boolean DEBUG = false;

    /**
     * It logs information.
     * @param text It is the text to log.
     */
    static final void log_i(String text) {
        if (DEBUG) Log.i(TAG, text);
    }

    /**
     * It logs warnings.
     * @param text It is the text to log.
     */
    static final void log_w(String text) {
        if (DEBUG) Log.w(TAG, text);
    }

    /**
     * It logs errors.
     * @param text It is the text to log.
     */
    static final void log_e(String text) {
        if (DEBUG) Log.e(TAG, text);
    }

    /**
     * It is current sdsp interface.
     */
    private SdspManager mSdspManager = null;

    /**
     * It is the SDSP synchronize object.
     */
    private Object mSdspSync = null;

    /**
     * It is the user callback.
     */
    NetUserCallback mNetUserCallback = null; 

    /**
     * It is the package call back.
     */
    NetPackageCallback mNetPackageCallback = null;

    /**
     * It is the net user mananger.
     */
    NetUserManager mNUMgr = null;

    /**
     * It is the net data receiver mananger.
     */
    NetDataReceiverManager mNDRMgr = null;

    /**
     * It is the http media server.
     */
    private MediaServer mMediaServer = null;

    /**
     * It is the flag indicate if the sdsp module is start.
     */
    private boolean mSdspStart = false;

    /**
     * It is the net resurrection.
     */
    private NetResurrection mNetResurrection = null; 

    /**
     * It is the context.
     */
    Context mContext = null;

    /**
     * It is the connectivity center.
     */
    private ConnectivityCenter mCC = null;

    /**
     * It is the handler to process user connect event.
     */
    NetHandler mHandler = null;

    /**
     * It is the main thread handler.
     */
    private Handler mMainHandler = null;

    /**
     * It is the multicast address.
     */
    private String mMcAddr = null;

    /**
     * It is the name of the net component.
     */
    private String mName = null;

    /**
     * It is the UID of the net component. In SDSP, it is limited by 16 byte long.
     */
    private String mUID;

    /**
     * It is the SVC ID of the net component. In SDSP, it is limited by 16 byte long.
     */
    private String mSVCID;

    /**
     * It is the preparing flag.
     */
    private boolean mPreparing =false;

    /**
     * It is the destroy flag.
     */
    private boolean mDestroyed = false;

    /**
     * It is the multicast lock to allow muticast packages.
     */
    private WifiManager.MulticastLock mMulticastLock = null;

    /**
     * It is the net user listener.
     */
    private NetUserListener mNUL;

    /**
     * It is the start run process;
     */
    private Runnable mStartRun = new Runnable() {
        public void run() {
            mPreparing = true;
            mMainHandler.removeCallbacks(this);

            synchronized(mSdspSync) {
                log_i("-- Start SDSP!");
                int port = 0;
                if (mNetResurrection != null) {
                    port = mNetResurrection.resurrect(mName, mUID, mSVCID);
                }

                log_i("-- Request Port:" + port +"!");
                mSdspManager = SdspManager.createManager(port);
                if (mUID != null) {
                    mSdspManager.setUUID(mUID.getBytes());
                } else {
                    mSdspManager.setUUID(
                        Util.longToByteArray((long) (Math.random() * Long.MAX_VALUE)));
                }
                mSdspManager.setClientCallback(mNetUserCallback);
                mSdspManager.setPacketCallback(mNetPackageCallback);
                mSdspManager.setMultiCastGroup(mMcAddr);
    
                if (mName == null) {
                    mSdspManager.setFriendName(Build.MODEL);
                } else {
                    mSdspManager.setFriendName(mName);
                }

                mSdspManager.setSVCID(mSVCID.getBytes());

                port = mSdspManager.start();

                if (port <= 0) {
                    if (mNetResurrection != null) {
                        mNetResurrection.giveUp(mName, mUID, mSVCID);
                    }
                    log_w("-- Illegal Port:" + port +"!");
                    log_e("-- Stop!");
                    stop();
                } else if ((port < 1024) || (port > 65535)) {
                    if (mNetResurrection != null) {
                        mNetResurrection.giveUp(mName, mUID, mSVCID);
                    }
                    log_w("-- Illegal Port:" + port +"!");
                    stop();

                    log_i("-- Shedule next start process!");
                    mMainHandler.postDelayed(this, 1359);
                } else {
                    log_i("-- Use Port:" + port +"!");
                    if (mNetResurrection != null) {
                        mNetResurrection.admit(mName, mUID, mSVCID, port);
                    }
                    mSdspStart = true;
                }
                mPreparing = false;
            }
        }
    };

    /**
     * It is the flag to use the stable port.
     */
    public static final int FLAG_USE_STABLE_PORT = 0x80000000;

    /**
     * It constructs the net manager controller. A net manager works as a network interface with
     * OOSA3 technology.
     * @param context It is the context of a application.
     * @param flag It is the flag of the net manager.
     * @param mcAddr It is the multicast address. This parameter is only available in sdsp
     * protocol. <BR>
     * Its range starts from 224.0.0.0 to 239.255.255.255.<BR>
     * The range starts from 224.0.0.0 to 224.0.0.255 is the local network address.<BR>
     * The range starts from 224.0.1.0 to 238.255.255.255 is internet address.<BR>
     * The range starts form 239.0.0.0 to 239.255.255.255 is manage address.(Normally do not use)
     * @param name It is the name of the network component.
     * @param nul It is the net user listener to process net user connect, update and disconnect
     * events.
     * @param idName It is the name to see others. All network components start with the name of
     * idName will been seen by this network component. 
     * @param npl It is the net package listener to process data packages.
     * @param uid <I>(This parameter has not worked yet!!)</I>
     * <BR>It is the UID of the network component. Generally, it is the identifier of the device
     * which runs the network component.
     * @param svcid <I>(This parameter has not worked yet!!)</I>
     * <BR>It is the service id of the network component. A service id is used to identify a
     * network application from another one.
     * @param doNotSeeNames It is the name sets that do not want to be seen. Any network
     * components with the name in this set will be filtered.
     */
    public NetManager(
            Context context,
            int flag,
            String mcAddr,
            String name,
            NetUserListener nul,
            String idName,
            NetPackageListener npl,
            String uid,
            String svcid,
            String ... doNotSeeNames) {
        // +<<---------------------+
        // | Log the onet version. |
        // +-----------------------+
        log_i("---------------------------------------------------");
        log_i("-- [onet] Version: 34 | Release Date: 2013-05-09 --");
        log_i("-- CHEN JIAN WEN                                 --");
        log_i("---------------------------------------------------");

        mContext = context;

        mMainHandler = new Handler(context.getMainLooper());

        mSVCID = svcid;
        mName = name;
        mUID = uid;
        log_i("-- Name: " + mName);
        log_i("-- SVCID: " + mSVCID);
        log_i("-- UID: " + mUID);
        mSdspSync = new Object();

        // Initialize the net resurrection.
        if ((FLAG_USE_STABLE_PORT & flag) != 0) {
            log_i("-- USE STABLE PORT");
            mNetResurrection = new NetResurrection(context);
        } else {
            mNetResurrection = null;
        }

        // Initialize the net package receiver mananger.
        mNDRMgr = new NetDataReceiverManager();

        // Initialize the net user mananger;
        mNUMgr = new NetUserManager();

        // Initialize the callback of net user.
        mNUL = nul;
        mHandler = new NetHandler(mNUMgr, nul, idName, doNotSeeNames);
        mNetUserCallback = new NetUserCallback(mHandler);

        // Initialize the callback of net package.
        mNetPackageCallback = new NetPackageCallback(mNUMgr, mNDRMgr, npl);

        // Initialize the multicast lock to allow the multicast be received.
        final WifiManager wm = (WifiManager) context.getSystemService(Context.WIFI_SERVICE);
        mMulticastLock = wm.createMulticastLock(this.getClass().getName());
        mMulticastLock.acquire();

        if (mcAddr == null) {
            mMcAddr = "224.0.0.0";
        } else {
            mMcAddr = mcAddr;
        }
        log_i("-- Multicast Address: " + mMcAddr);

        // Register connectivity receiver.
        mCC = new ConnectivityCenter(context);
        mCC.registerListener(new NetConnectivityListener(this));

        if (idName != null) {
            log_i("-- Specified remote client name start with: " + idName);
        } else {
            log_i("-- No remote client name specified!" );
        }
        log_i("-- Skip following clients");
        for (String p : doNotSeeNames) {
            log_i(p);
        }
        log_i("--------------------------");
    }

    /**
     * It stops the network manager.
     */
    void stop() {
        if (mDestroyed) return;
        synchronized(mSdspSync) {
            if (mSdspStart) {
                mSdspStart = false;
                mSdspManager.stop();
                mSdspManager = null;

                mNUMgr.clear();
                if (mNUL != null) {
                    mNUL.onClear();
                }
            }
        }
    }

    /**
     * It refreshes the connection of net component. It makes other know it exists.
     */
    public void refreshConnections() {
        if (mDestroyed) return;
        synchronized(mSdspSync) {
            if (!mSdspStart) {
                if (!mPreparing) {
                    mPreparing = true;
                    log_i("-- Prepare to open net work!");
                    // Just delay about 900ms.
                    mMainHandler.postDelayed(mStartRun, 900);
                } else {
                    log_i("-- It is opening net work!");
                }
            } else {
                log_i("-- Refresh the interface!");
                mSdspManager.refreshInterface();
                mSdspManager.sendSearchMessage();
            }
        }
    }

    /**
     * It gets the net user manager.
     * @return It returns the net user manager.
     */
    public NetUserManager getUserManager() {
        if (mDestroyed) return null;
        return mNUMgr;
    }

    /**
     * It gets the net data receiver manager.
     * @return It returns the net package receiver manager.
     */
    public NetDataReceiverManager getNetDataReceiverManager() {
        if (mDestroyed) return null;
        return mNDRMgr;
    }

    /**
     * It gets the media server.
     * @return It returns the http media server.
     */
    public MediaServer getMediaServer() {
        if (mDestroyed) return null;
        return mMediaServer;
    }

    /**
     * It gets the connectivity center.
     * @return It returns the connectivity center.
     */
    public ConnectivityCenter getConnectivityCenter() {
        if (mDestroyed) return null;
        return mCC;
    }

    /**
     * It destroys the net component.
     */
    public void destroy() {
        log_i("-- Destroy !");
        mDestroyed = true;

        // Clear start run process.
        mMainHandler.removeCallbacks(mStartRun);

        mCC.destroy();

        synchronized(mSdspSync) {
            if (mSdspManager != null) {
                mSdspManager.stop();
                mSdspManager= null;
                mSdspStart = false;
            }
        }

        if (mMediaServer != null) {
            mMediaServer.stop();
            mMediaServer = null;
        }

        if (mNDRMgr != null) {
            mNDRMgr.destroy();
            mNDRMgr = null;
        }

        // Release muticast lock.
        if (mMulticastLock != null) {
            mMulticastLock.release();
            mMulticastLock = null;
        }
    }

    /**
     * It sends package to remote device.
     * @param pkg It is the package to send out.
     * @param client It is the client to send to.
     */
    public void sendPackage(DataPackage pkg, NetUser user) {
        if (mDestroyed) return;
        if ((pkg == null) || (user == null)) {
            return;
        }
        byte buf[] = null;
        boolean encrypt = false;
        final int type = pkg.getType();
        if (type == DataPackage.PKG_TYPE_STRING) {
            buf = ((StringPackage)pkg).write();
            if (buf != null) {
                ((StringPackage)pkg).writePkgHeader(buf);
            }
        } else if (type == DataPackage.PKG_TYPE_BYTE_BUFFER) {
            buf = ((ByteBufferPackage)pkg).write();
            if (buf != null) {
                ((ByteBufferPackage)pkg).writePkgHeader(buf);
            }
            encrypt = true;
        } else if (type == DataPackage.PKG_TYPE_KEY_EVENT) {
            buf = ((KeyEventPackage)pkg).write();
            if (buf != null) {
                ((KeyEventPackage)pkg).writePkgHeader(buf);
            }
        } else if (type == DataPackage.PKG_TYPE_MOTION_EVENT) {
            buf = ((MotionEventPackage)pkg).write();
            if (buf != null) {
                ((MotionEventPackage)pkg).writePkgHeader(buf);
            }
        } else if (type == DataPackage.PKG_TYPE_FULL_MOTION_EVENT) {
            buf = ((FullMotionEventPackage)pkg).write();
            if (buf != null) {
                ((FullMotionEventPackage)pkg).writePkgHeader(buf);
            }
        } else if (type == DataPackage.PKG_TYPE_SENSOR_EVENT) {
            buf = ((SensorEventPackage)pkg).write();
            if (buf != null) {
                ((SensorEventPackage)pkg).writePkgHeader(buf);
            }
        } else {
            log_w("-- Unsupported package type!! Contact with JIAN WEN!");
        }
        if (buf != null) {
            synchronized(mSdspSync) {
                if (mSdspStart) {
                    if (encrypt) {
                        mSdspManager.sendData(buf, user.getClient(), true);
                    } else {
                        mSdspManager.sendData(buf, user.getClient());
                    }
                }
            }
        }
    }

    /**
     * It sets the nick name of the net component.
     * @param name It is the nick name to set.
     */
    public void setNickName(String name) {
        if (mDestroyed) return;
        synchronized(mSdspSync) {
            mName = name;
            if (name == null) name = Build.MODEL;
            if (mSdspStart) {
                mSdspManager.setFriendName(name);
                mSdspManager.sendSearchMessage();
            }
        }
    }

    public boolean callActivity(int action, String cmd, NetUser user) {
        if (mDestroyed) return false;
        if (mNUMgr.size() == 0)
            return true;
        DataPackage dpk = DataPackageFactory
                    .peekDataPackage(DataPackage.PKG_TYPE_STRING);
            dpk.setString(cmd);
            dpk.setAction(action);

        log_i("Call Activity:" + cmd);
        sendPackage(dpk, user);
        return true;
    }

    /**
     * It helps to create the media server in the net manager module.
     * If the media server already exists, this method does nothing.
     */
    public void createMediaServer() {
        if (mDestroyed) return;
        if (mMediaServer == null) {
            mMediaServer = new MediaServer();
        }
    }

    /**
     * It enable or disable the debug of onet module.
     * @param debug It enable the debug when it is true and disable debug when itis false.
     */
    public static void enableDebug(boolean debug) {
        DEBUG = debug;
    }

    /**
     * It resets the net resurrection to obtain another instance of net component.
     */
    public void resetNetResurrection() {
        if (mDestroyed) return;
        if (mNetResurrection != null) {
            mNetResurrection.giveUp(mName, mUID, mSVCID);
        }
    }

    /**
     * It is the static field to load jni library.
     */
    static {
        try {
            System.loadLibrary("sharebox_sdsp_jni");
            
        } catch (UnsatisfiedLinkError e1) {
            System.err.println("WARNING: Could not load so.");
            try {
                System.loadLibrary("launcher_jni");
            } catch (UnsatisfiedLinkError e2) {
                System.err.println("WARNING: Could not load jni so in system, too.");
            }
        }
    }
}
