package com.oozic.net;

import android.content.Context;
import android.content.SharedPreferences;

/**
 * Net resurection is used to make the net component resurrect. In another word, it makes the new
 * net component have no change with the last one.
 * @author CHEN JIAN WEN
 *
 */
class NetResurrection {
    /**
     * It is the tag of this class.
     */
    private static final String TAG = "NetResurrection";

    /**
     * It is the key to store the TCP port information.
     */
    private static final String KEY_PORT = "key:port";

    /**
     * It is the context of activity or service.
     */
    private Context mContext;

    /**
     * It is the synchronize object.
     */
    private Object mSync;

    /**
     * It constructs the net resurrection.
     * @param context It is the context of the activity or service.
     */
    public NetResurrection(Context context) {
        mContext = context;
        mSync = new Object();
    }

    /**
     * It admits the net component to resrrect.
     * @param name It is the name of the net component.
     * @param uid It is the uid of the net component.
     * @param svcid It is the svcid of the net component.
     * @param port It is the TCP port of the net component.
     */
    public void admit(String name, String uid, String svcid, int port) {
        synchronized(mSync) {
            final SharedPreferences sp = getSharedPrefereces(name, uid, svcid);
            final SharedPreferences.Editor edt = sp.edit();
            edt.putInt(KEY_PORT, port);
            edt.commit();
        }
    }

    /**
     * It resurrect the net component by return special TCP port that is used last time.
     * @param name It is the name of the net component.
     * @param uid It is the uid of the net component.
     * @param svcid It is the svcid of the net component.
     * @return It returns the TCP port that is used last time. Its range is from 1025 to 65535. If
     * the port value is out of range, a random port in range will be used (0).
     */
    public int resurrect(String name, String uid, String svcid) {
        int port  = 0;
        synchronized(mSync) {
            final SharedPreferences sp = getSharedPrefereces(name, uid, svcid);
            port = sp.getInt(KEY_PORT, 0);
        }
        if (port < 1024 || port > 65535) port = 0;
        return port;
    }

    /**
     * It gives up the net component.
     * @param name It is the name of the net component.
     * @param uid It is the uid of the net component.
     * @param svcid It is the svcid of the net component.
     */
    public void giveUp(String name, String uid, String svcid) {
        synchronized(mSync) {
            final SharedPreferences sp = getSharedPrefereces(name, uid, svcid);
            final SharedPreferences.Editor edt = sp.edit();
            edt.clear();
            edt.commit();
        }
    }

    /**
     * It helps to get the shared prefereces object by name, uid and svcid.
     * @param name It is the name of the net component.
     * @param uid It is the uid of the net component.
     * @param svcid It is the svcid of the net component.
     * @return It returns the shared prefereces object.
     */
    private SharedPreferences getSharedPrefereces(String name, String uid, String svcid) {
        if (name == null) name = "";
        if (uid == null) uid = "";
        if (svcid == null) svcid = "";
        final String spName = TAG + ":" + name + ":" + uid + ":" + svcid;
        
        return mContext.getSharedPreferences(spName, Context.MODE_PRIVATE);
    }
}