package com.oozic.net;

import java.util.ArrayList;

import android.util.Log;

import com.oozic.library.sdsp.SdspClient;
import com.oozic.library.sdsp.SocketAddress;

/**
 * It is used to manager the net user.
 * @author CHEN JIAN WEN 
 */
public class NetUserManager {
    /**
     * It is the log tag.
     */
    private static final String TAG = "NetUserManager";

    /**
     * It is the user set.
     */
    private ArrayList<NetUser> mUsers = new ArrayList<NetUser>();

    /**
     * It is the debug switch.
     */
    static final boolean DEBUG = NetManager.DEBUG;

    /**
     * It logs information.
     * @param text It is the text to log.
     */
    final void log_i(String text) {
        if (DEBUG) Log.i(TAG, text);
    }

    /**
     * It logs warnings.
     * @param text It is the text to log.
     */
    final void log_w(String text) {
        if (DEBUG) Log.w(TAG, text);
    }

    /**
     * It logs errors.
     * @param text It is the text to log.
     */
    final void log_e(String text) {
        if (DEBUG) Log.e(TAG, text);
    }

    NetUserManager() {
    }

    /**
     * It clears the user in the manager.
     */
    public void clear() {
        synchronized (mUsers) {
            mUsers.clear();
        }
    }

    public int size() {
        int count = 0;
        synchronized (mUsers) {
            count = mUsers.size();
        }
        return count;
    }

    /**
     * It add the net user to mananger.
     * @param nu It is the user to add.
     * @return It returns the status of adding user.
     * <BR> 0 User is new added.
     * <BR> 1 User exists, update.
     */
    public int add(final NetUser nu) {
        synchronized (mUsers) {
            int ret = 0;
            for (NetUser p : mUsers) {

                if (nu.equals(p)) {
                    p.updateNetUser(nu);
                    log_i("Replace!" + p.mName);
                    return ret + 1;
                }
            }
            log_i("Add New!" + nu.mName);
            mUsers.add(nu);
            return ret;
        }
    }

    /**
     * It removes the net user from net user manager.
     * @param client It is the user of new net interface.
     * @return It returns the net user that is removed from mananger. If the client is not found or
     * the user is not expecting to be removed, it returns null.
     */
    public NetUser remove(int key, final NetUserListener nul) {
        synchronized (mUsers) {
            if (key != 0) {
                for (NetUser p : mUsers) {
                    if (p.getKey() == key) {
                        log_i("Remove! " + p.mAccount + ":" + p.mName);
                        mUsers.remove(p);
                        p.mKey = 0;
                        return p;
                    }
                }
            }
        }
        return null;
    }

    /**
     * It updates the net user.
     * @param client It is the user of new net interface.
     * @return It returns the net user that is updated.
     */
    public NetUser update(SdspClient client) {
        log_i("update " + client.getName());
        synchronized (mUsers) {
            if (client != null) {
                for (NetUser p : mUsers) {
                    if (p.equals(client)) {
                        p.updateClient(client);
                        return p;
                    }
                }
            }
        }
        return null;
    }

    public String getName(int deviceIndex) {
        String name = null;
        synchronized (mUsers) {
            final int count = mUsers.size();

            if (deviceIndex >= count) {
                return null;
            }

            name = mUsers.get(deviceIndex).mName;
        }
        return name;
    }

    /**
     * It gets the net user with its index.
     * @param index It is the index of the user.
     * @return It returns a net user.
     */
    public NetUser get(int index) {
        return mUsers.get(index);
    }

    /**
     * It helps to get the net user.
     * @param client It is the client of the user.
     * @return It returns a net user if success. It returns null otherwise.
     */
    public NetUser get(SdspClient client) {
        SocketAddress address = client.getAddress();
        if(address == null){
            return null;
        }

        synchronized (mUsers) {
            if (client != null) {
                for (NetUser i : mUsers) {
                    
                    if(i.equals(address)){
                        return i;
                    }
                }
            }
        }
        return null;
    }

    /**
     * It helps to get the net user.
     * @param ip It is the ip of the user to get.
     * @param port It is the port of the user to get.
     * @return It returns a net user if sucess. It retuns null otherwise.
     */
    public NetUser get(String ip, int port) {
        if (ip == null) {
            return null;
        }

        synchronized (mUsers) {
            for (NetUser i : mUsers) {
                if(i.equals(ip, port)){
                    return i;
                }
            }
        }
        return null;
    }

    /**
     * It gets the synchornize object.
     * @return It returns the synchronize object.
     */
    public Object getSyncObject() {
        return mUsers;
    }
}
