package com.oozic.net.fdt;

import android.os.Environment;

import com.oozic.net.NetManager;
import com.oozic.net.NetUser;

/**
 * It is the file direct transceive module manager. A file direct transceive module helps to send a
 * file obejct to other net components or to receive a file from other net components.<BR>
 * It supports to send up to 8 files at the same time.
 * @author CHEN JIAN WEN
 *
 */
public class FDTManager {
    /**
     * It is the class tag.
     */
    private static final String TAG = "FDTManager";

    /**
     * It is the time out error code.
     */
    public static final int ERROR_CODE_TIMEOUT = 0x0051CEC0;

    /**
     * It is the remote file io error.
     */
    public static final int ERROR_CODE_REMOTE_IO = 0x0051CEC1;

    /**
     * It is the local file io error.
     */
    public static final int ERROR_CODE_LOCAL_IO = 0x0051CEC2;

    /**
     * It is the cancel file transfer error.
     */
    public static final int ERROR_CODE_CANCEL = 0x0051CEC3;

    /**
     * It is the directory to receive the file.
     */
    String mDirectory = Environment.getExternalStorageDirectory().getAbsolutePath();

    /**
     * It is the net manager.
     */
    NetManager mNetManager;

    /**
     * It is the block size of file data.
     */
    int mBlockSize = 1024;

    /**
     * It is the FDT main thread.
     */
    private FDTThread mThread;

    /**
     * It is the file receiver manager.
     */
    private FileReceiverManager mFRM;

    /**
     * It constructs the file direct transfer manager.
     * @param net It is the net manager.
     * @param fal It is the file accept listener. If this parameter is set to null, the default
     * accept listener and receive listener will be used.
     */
    public FDTManager(NetManager nm, FileAcceptListener fal) {
        mNetManager = nm;
        mThread = new FDTThread(this);
        mThread.start();
        if (fal == null) {
            // Make a default file accept listener.
            fal = new FileAcceptListener() {

                @Override
                public boolean isAccepted() {
                    // The file is accepted immediantly.
                    return true;
                }

                @Override
                public boolean isOnHold() {
                    // It should not hold the request any more.
                    return false;
                }

                @Override
                public void onStart(FileReceiver fr) {
                    // Set the default file receive listener to receiver.
                    fr.setFileReceiveListener(new FileReceiveListener() {

                        @Override
                        public void onComplete(String path) {
                            android.util.Log.e(TAG, "receive complete! " + path);
                        }

                        @Override
                        public void onDataReceived(float per) {
                            android.util.Log.e(TAG, "receive:" + String.format("%s", per));
                        }

                        @Override
                        public void onError(int errorCode) {
                            android.util.Log.e(TAG, "receive error!" + errorCode);
                        }
                        
                    });
                }

                @Override
                public void onCancel() { 
                }

            };
        }
        mFRM = new FileReceiverManager(this, fal);
    }

    /**
     * It sets the file receive directory.
     * @param dir It is the file receive directory to set. If null pointer is set, the default
     * directory willl be used.
     */
    public void setReceiveDirectory(String dir) {
        if (dir == null) {
            mDirectory = Environment.getExternalStorageDirectory().getAbsolutePath();
        } else {
            mDirectory = dir;
        }
    }

    /**
     * It gets the file receive directory.
     * @return It returns the file receive directory. 
     */
    public String getReceiveDirectory() {
        return mDirectory;
    }

    /**
     * It destroy the FDT manager. 
     */
    public void destroy() {
        if (mThread != null) {
            mThread.quit();
            mThread = null;
        }
        if (mFRM != null) {
            mFRM.destroy();
        }
    }

    /**
     * It sends a file to 
     * @param path It is the file path.
     * @param target It is the target to send file.
     * @param fsl It is the file send listener that calls by FDT in the whoel file send process.
     */
    public void sendFile(String path, NetUser target, FileSendListener fsl) {
        if (mThread != null) {
            mThread.sendFile(path, target, fsl);
        }
    }

    /**
     * It sets the block size.
     * @param size It is the block size to receive.
     */
    public void setBlockSize(int size) {
        if (size <= 0) size = 1024;
        mBlockSize = size;
    }
}
