package com.oozic.net.fdt;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.RandomAccessFile;

import com.oozic.net.ByteBufferPackage;
import com.oozic.net.DataPackage;
import com.oozic.net.DataPackageFactory;
import com.oozic.net.NetData;
import com.oozic.net.NetDataReceiverListener;
import com.oozic.net.NetDataReceiverManager;
import com.oozic.net.NetManager;
import com.oozic.net.NetUser;

/**
 * It is the receiver to receive the file transfer request.
 * @author CHEN JIAN WEN
 *
 */
public class FileReceiver implements NetDataReceiverListener{
    /**
     * It is the file direct transfer manager.
     */
    private FDTManager mFDTM;

    /**
     * It is the index array.
     */
    private int[] mIndexes;

    /**
     * It is the file.
     */
    private File mFile;

    /**
     * It is the file writer.
     */
    private RandomAccessFile mRAFile;

    /**
     * It is the block size of the receiver.
     */
    private int mBlockSize;

    /**
     * It is the command key.
     */
    private int mCmdKey;

    /**
     * It is the data key.
     */
    private int mDataKey;

    /**
     * It is file receive listener.
     */
    private FileReceiveListener mFRL;

    /**
     * It constructs the file receiver.
     * @param fdtm It is the file direct transfer manager.
     * @param key It is the key to reply.
     * @param path It is the file path.
     * @param size It is the file size.
     * @param blockSize It is the block size.
     */
    FileReceiver(FDTManager fdtm, int key, NetUser nu, String path, long size, int blockSize) {
        mFRL = null;
        mFDTM = fdtm;
        mBlockSize = blockSize;
        mCmdKey = key;
        final NetManager nm = mFDTM.mNetManager;
        // Get file.
        final String name = path.substring(path.lastIndexOf("/") + 1);
        File file;
        int i = 0;

        do {
            final String fn;
            if (i == 0) {
                fn = fdtm.mDirectory + "/" + name;
            } else {
                fn = fdtm.mDirectory + "/(" + i + ")" + name;
            }
 
            file = new File(fn);
            i++;
        } while (file.exists());

        try {
            file.createNewFile();
        } catch (IOException e) {
            e.printStackTrace();
            // Cannot create the file.
            // Send remote space error back.
            final Data_SendFileErr err =
                new Data_SendFileErr(FDTManager.ERROR_CODE_REMOTE_IO);
            err.setKey(key);
            final DataPackage pkg =
                DataPackageFactory.peekDataPackage(DataPackage.PKG_TYPE_BYTE_BUFFER);
            pkg.setAction(DataPackage.ACTION_REPLY);
            pkg.setByteBuffer(err.toBytes());
            nm.sendPackage(pkg, nu);
            return;
        }

        try {
            mRAFile = new RandomAccessFile(file, "rw");
            mFile = file;
        } catch (FileNotFoundException e) {
            // It cannot be here.
            e.printStackTrace();
            return;
        }

        final NetDataReceiverManager ndrm = nm.getNetDataReceiverManager();
        mDataKey = ndrm.createKey(DataPackage.ACTION_INFORM, NetData.ID_FILE_TRANSCEIVE_DATA);

        ndrm.createReceiver(
                DataPackage.ACTION_INFORM,
                NetData.ID_FILE_TRANSCEIVE_DATA,
                mDataKey, this, false);

        // Reply to the one who send the reqeust.
        final int l = ((size == 0) ? 0 : ((int)(size / blockSize) + 1));
        mIndexes = new int[l];
        i = 0;
        for (i = 0; i < l; i++) {
            mIndexes[i] = i;
        }

        final Data_SendFileAck ack =
            new Data_SendFileAck(blockSize, mDataKey, mIndexes);
        ack.setKey(mCmdKey);
        final DataPackage pkg = DataPackageFactory.peekDataPackage(DataPackage.PKG_TYPE_BYTE_BUFFER);
        pkg.setAction(DataPackage.ACTION_REPLY);
        pkg.setByteBuffer(ack.toBytes());
        nm.sendPackage(pkg, nu);
    }

    @Override
    public void run(int key, ByteBufferPackage pack, NetUser nu) {
        // Check the package.
        final Data_SendFileDat dat = new Data_SendFileDat();
        if (!dat.fromBytes(pack.getByteBuffer())) {
            return;
        }
        final int index = dat.getDataIndex();
        if (index < mIndexes.length) {
            if (mFRL != null) {
                mFRL.onDataReceived((float)index / (float)mIndexes.length);
            }
            if (mIndexes[index] >= 0) {
                final NetManager nm = mFDTM.mNetManager;
                try {
                    mRAFile.seek(index * mBlockSize);
                    mRAFile.write(dat.getData());
                    mIndexes[index] = -1;
                } catch (IOException e) {
                    e.printStackTrace();

                    // Send remote space error back.
                    final Data_SendFileErr err =
                        new Data_SendFileErr(FDTManager.ERROR_CODE_REMOTE_IO);
                    err.setKey(key);
                    final DataPackage pkg =
                        DataPackageFactory.peekDataPackage(DataPackage.PKG_TYPE_BYTE_BUFFER);
                    pkg.setAction(DataPackage.ACTION_REPLY);
                    pkg.setByteBuffer(err.toBytes());
                    nm.sendPackage(pkg, nu);
                    
                    final NetDataReceiverManager ndrm = nm.getNetDataReceiverManager();
                    if (ndrm != null) {
                        ndrm.kill(DataPackage.ACTION_INFORM,
                                NetData.ID_FILE_TRANSCEIVE_DATA, mDataKey);
                    }

                    try {
                        mRAFile.close();
                    } catch (IOException e1) {
                        e1.printStackTrace();
                    }

                    mFile.delete();

                    if (mFRL != null) {
                        mFRL.onError(FDTManager.ERROR_CODE_LOCAL_IO);
                    }
                    return;
                }
                boolean finish = true;
                for (int i = index; i < mIndexes.length; i++) {
                    if (mIndexes[i] != -1) {
                        finish = false;
                        break;
                    }
                }

                boolean compelte = false;

                if (finish) {
                    compelte = true;
                    final Data_SendFileAck ack =
                        new Data_SendFileAck(mBlockSize, mDataKey, mIndexes);
                    ack.setKey(mCmdKey);
                    final DataPackage pkg = DataPackageFactory.peekDataPackage(DataPackage.PKG_TYPE_BYTE_BUFFER);
                    pkg.setAction(DataPackage.ACTION_REPLY);
                    pkg.setByteBuffer(ack.toBytes());
                    nm.sendPackage(pkg, nu);

                    for (int i = index; i < mIndexes.length; i++) {
                        if (mIndexes[i] != -1) {
                            compelte = false;
                            break;
                        }
                    }

                    if (compelte) {
                        try {
                            mRAFile.close();
                        } catch (IOException e) {
                            e.printStackTrace();
                        }
    
                        if (mFRL != null) {
                            mFRL.onComplete(mFile.getAbsolutePath());
                        }
                    }
                }

                final NetDataReceiverManager ndrm = nm.getNetDataReceiverManager();
                if (ndrm != null) {
                    if (compelte) {
                        ndrm.kill(DataPackage.ACTION_INFORM,
                                NetData.ID_FILE_TRANSCEIVE_DATA, mDataKey);
                    } else {
                        ndrm.touch(DataPackage.ACTION_INFORM,
                                NetData.ID_FILE_TRANSCEIVE_DATA, mDataKey);
                    }
                }
            }
        }
    }

    @Override
    public void onTimeOut(int action, int id, int key) {
        // If this method is called.
        try {
            mRAFile.close();
        } catch (IOException e) {
            e.printStackTrace();
        }

        mFile.delete();

        if (mFRL != null) {
            mFRL.onError(FDTManager.ERROR_CODE_TIMEOUT);
        }
    }

    /**
     * It sets the file receive listener.
     * @param frl It is the file receive listener to set.
     */
    public void setFileReceiveListener(FileReceiveListener frl) {
        mFRL = frl;
    }
}
