package com.oozic.net.fdt;

import java.io.File;

import android.os.Handler;
import android.os.Looper;

import com.oozic.net.ByteBufferPackage;
import com.oozic.net.DataPackage;
import com.oozic.net.DataPackageFactory;
import com.oozic.net.NetData;
import com.oozic.net.NetDataReceiverListener;
import com.oozic.net.NetDataReceiverManager;
import com.oozic.net.NetManager;
import com.oozic.net.NetUser;

/**
 * It is the file sender.
 * @author CHEN JIAN WEN.
 *
 */
class FileSender {

    /**
     * It is the FDT manager.
     */
    private FDTManager mFDTM;

    /**
     * It is the file sender thread to send file data blocks.
     */
    private FileSenderThread mFileSenderThread;

    /**
     * It is the file sender manager.
     */
    private FileSenderManager mFSM;

    /**
     * It contructs the file sender.
     * @param fdtm It is the FDT manager.
     * @param fsm It is the file sender manager.
     */
    FileSender(FDTManager fdtm, FileSenderManager fsm) {
        mFDTM = fdtm;
        mFileSenderThread = null;
        mFSM = fsm;
    }

    /**
     * It sends the file to 
     * @param fst It is file send task.
     */
    boolean send(final FileSendTask fst) {
        if (mFSM == null) return false;
        if (fst == null) return false;
        if (fst.mPath == null) return false;
        if (fst.mTarget == null) return false;

        final File file = new File(fst.mPath);

        if (!file.exists()) {
            if (fst.mListener != null) {
                final Handler h = new Handler(Looper.getMainLooper());
                h.post(new Runnable() {
                    public void run() {
                        fst.mListener.onError(FDTManager.ERROR_CODE_LOCAL_IO);
                    }
                });
            }
            return false;
        }


        final NetManager nm = mFDTM.mNetManager;
        final NetDataReceiverManager ndrm = nm.getNetDataReceiverManager();
        // Create the key. A Key must create before the receiver register in.
        final int k = ndrm.createKey(DataPackage.ACTION_REPLY, NetData.ID_FILE_TRANSCEIVE_ACK);

        // Create the ack receiver.
        ndrm.createReceiver(DataPackage.ACTION_REPLY,
                NetData.ID_FILE_TRANSCEIVE_ACK,
                k,
                new NetDataReceiverListener() {

                    @Override
                    public void run(int key, ByteBufferPackage pack, NetUser nu) {
                        if (!nu.equals(fst.mTarget)) return;

                        final Data_SendFileAck ack = new Data_SendFileAck();
                        if (!ack.fromBytes(pack.getByteBuffer())) return;

                        final int size = ack.getFileBlockSize();
                        final int[] indexes = ack.getFileBlockIndexes();
                        final int dataKey = ack.getDataKey();


                        // Start the send thread.
                        if (mFileSenderThread != null) {
                            if (mFileSenderThread.isAlive()) {
                                mFileSenderThread.interrupt();
                            }
                        }

                        if (size == -1) {
                            ndrm.touch(DataPackage.ACTION_REPLY, NetData.ID_FILE_TRANSCEIVE_ACK, key);
                            ndrm.touch(DataPackage.ACTION_REPLY, NetData.ID_FILE_TRANSCEIVE_ERROR, key);
                            return;
                        } else if (size < -1){
                            // Transfer is cancelled. Kill the receivers.
                            ndrm.kill(DataPackage.ACTION_REPLY, NetData.ID_FILE_TRANSCEIVE_ACK, key);
                            ndrm.kill(DataPackage.ACTION_REPLY, NetData.ID_FILE_TRANSCEIVE_ERROR, key);
                            if (fst.mListener != null) {
                                final Handler h = new Handler(Looper.getMainLooper());
                                h.post(new Runnable() {
                                    public void run() {
                                        fst.mListener.onError(FDTManager.ERROR_CODE_CANCEL);
                                    }
                                });
                            }
                            mFSM.remove(FileSender.this);
                            return;
                        }

                        boolean finish = true;
                        for (int p : indexes) {
                            if (p != -1) {
                                finish = false;
                                break;
                            }
                        }

                        if (finish || (size == 0) || (indexes.length == 0)) {
                            // Transfer is finished. Kill the receivers.
                            ndrm.kill(DataPackage.ACTION_REPLY, NetData.ID_FILE_TRANSCEIVE_ACK, key);
                            ndrm.kill(DataPackage.ACTION_REPLY, NetData.ID_FILE_TRANSCEIVE_ERROR, key);
                            if (fst.mListener != null) {
                                final Handler h = new Handler(Looper.getMainLooper());
                                h.post(new Runnable() {
                                    public void run() {
                                        fst.mListener.onDataSended(1.0f);
                                        fst.mListener.onComplete();
                                    }
                                });
                            }
                            mFSM.remove(FileSender.this);
                            return;
                        }

                        mFileSenderThread = new FileSenderThread(mFDTM, fst.mTarget, fst.mPath,
                            size, key, dataKey, indexes, fst.mListener);
                        mFileSenderThread.start();

                        ndrm.touch(DataPackage.ACTION_REPLY, NetData.ID_FILE_TRANSCEIVE_ACK, key);
                        ndrm.touch(DataPackage.ACTION_REPLY, NetData.ID_FILE_TRANSCEIVE_ERROR, key);
                    }

                    @Override
                    public void onTimeOut(int action, int id, int key) {
                        // If this reached, this means the target does not reply anymore or there
                        // is a error on send thread. We need call the onError method of file send
                        // listener.
                        if (fst.mListener != null) {
                            final Handler h = new Handler(Looper.getMainLooper());
                            h.post(new Runnable() {
                                @Override
                                public void run() {
                                    fst.mListener.onError(FDTManager.ERROR_CODE_TIMEOUT);
                                }
                            });
                        }
                        // Make sure kill the error receiver.
                        ndrm.kill(DataPackage.ACTION_REPLY, NetData.ID_FILE_TRANSCEIVE_ERROR, key);
                        mFSM.remove(FileSender.this);
                    }
            
        }, false);

        // Create the error receiver.
        ndrm.createReceiver(DataPackage.ACTION_REPLY,
                NetData.ID_FILE_TRANSCEIVE_ERROR,
                k,
                new NetDataReceiverListener() {

                    @Override
                    public void run(int key, ByteBufferPackage pack, NetUser nu) {
                        if (!nu.equals(fst.mTarget)) return;

                        final Data_SendFileErr err = new Data_SendFileErr();
                        if (!err.fromBytes(pack.getByteBuffer())) return;

                        // If this reached, the target has problem on disk space or write file. 
                        if (fst.mListener != null) {
                            final Handler h = new Handler(Looper.getMainLooper());
                            h.post(new Runnable() {
                                @Override
                                public void run() {
                                    fst.mListener.onError(FDTManager.ERROR_CODE_REMOTE_IO);
                                }
                            });
                        }
                        // Kill both error receiver and acknowledge receiver.
                        ndrm.kill(DataPackage.ACTION_REPLY, NetData.ID_FILE_TRANSCEIVE_ACK, key);
                        ndrm.kill(DataPackage.ACTION_REPLY, NetData.ID_FILE_TRANSCEIVE_ERROR, key);
                        mFSM.remove(FileSender.this);
                    }

                    @Override
                    public void onTimeOut(int action, int id, int key) {
                        // If this reached, this means the target does not reply anymore or there
                        // is a error on send thread. We need call the onError method of file send
                        // listener.
                        if (fst.mListener != null) {
                            final Handler h = new Handler(Looper.getMainLooper());
                            h.post(new Runnable() {
                                @Override
                                public void run() {
                                    fst.mListener.onError(FDTManager.ERROR_CODE_TIMEOUT);
                                }
                            });
                        }
                        // Make sure kill the acknowledge receiver.
                        ndrm.kill(DataPackage.ACTION_REPLY, NetData.ID_FILE_TRANSCEIVE_ACK, key);
                        mFSM.remove(FileSender.this);
                    }
            
        }, false);

        // Send send request.
        final DataPackage pack =
            DataPackageFactory.peekDataPackage(DataPackage.PKG_TYPE_BYTE_BUFFER);
        pack.setAction(DataPackage.ACTION_REQUEST);
        final Data_SendFileReq req = new Data_SendFileReq(fst.mPath);
        req.setKey(k);
        pack.setByteBuffer(req.toBytes());

        nm.sendPackage(pack, fst.mTarget);
        return true;
    }

    /**
     * It destroys the file sender manually.
     */
    void destroy() {
        if (mFileSenderThread != null) {
            if (mFileSenderThread.isAlive()) {
                mFileSenderThread.interrupt();
            }
        }
    }
}
