package com.oozic.net.fdt;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.RandomAccessFile;

import android.os.Handler;
import android.os.Looper;
import android.util.Log;

import com.oozic.net.DataPackage;
import com.oozic.net.DataPackageFactory;
import com.oozic.net.NetData;
import com.oozic.net.NetDataReceiverManager;
import com.oozic.net.NetManager;
import com.oozic.net.NetUser;

/**
 * It is the thread to send the file data. This thread can be stop by interrupt.
 * @author CHEN JIAN WEN.
 *
 */
class FileSenderThread extends Thread {

    /**
     * It is the tag of the class.
     */
    private static final String TAG = "FileSenderThread";

    /**
     * It is the FDT manager.
     */
    private FDTManager mFDTM;

    /**
     * It is the target to send file to.
     */
    private NetUser mTarget;

    /**
     * It is the file path.
     */
    private String mPath;

    /**
     * It is the block size.
     */
    private int mBlockSize;

    /**
     * It is the command key.
     */
    private int mCmdKey;

    /**
     * It is the data key.
     */
    private int mDataKey;

    /**
     * It is the block indexes
     */
    private int[] mBlockIndexes;

    /**
     * It is the file send listener.
     */
    private FileSendListener mFSL;

    /**
     * It constructs the file sender thread.
     * @param fdtm It is FDT manager.
     * @param path It is the file path.
     * @param blockSize It is the file data block size.
     * @param cmdKey It is the command key to receive the result.
     * @param dataKey It is the data Key to send the file block.
     * @param blockIndexes It is the file block indexes.
     */
    FileSenderThread(FDTManager fdtm, NetUser target, String path,
            int blockSize, int cmdKey, int dataKey, int[] blockIndexes,
            FileSendListener listener) {
        super();

        mFDTM = fdtm;
        mTarget = target;
        mPath = path;
        mBlockSize = blockSize;
        mCmdKey = cmdKey;
        mDataKey = dataKey;
        mBlockIndexes = blockIndexes;
        mFSL = listener;
    }

    @Override
    public void run() {
        if ((mFDTM == null) || (mFDTM.mNetManager == null) ||
                (mTarget == null) ||
                (mPath == null) || (mBlockSize <= 0) ||
                (mBlockIndexes == null) || (mBlockIndexes.length == 0)) {
            // The parameters are illegal. 
            return;
        }
        final File file = new File(mPath);

        if (!file.exists()) {
            return;
        }

        RandomAccessFile raFile;
        try {
            raFile = new RandomAccessFile(file, "r");
        } catch (FileNotFoundException e) {
            // It should not be here.
            e.printStackTrace();
            return;
        }

        // Create the byte buffer.
        try {
            final byte[] buffer = new byte[mBlockSize];
            final NetManager nm = mFDTM.mNetManager;
            int i = 0;
            for (int p : mBlockIndexes) {
                try {
                    if (p < 0) {
                        i++;
                        final float per = (float)i / (float)mBlockIndexes.length;
                        if (mFSL != null) {
                            final Handler h = new Handler(Looper.getMainLooper());
                            h.post(new Runnable() {
                                public void run() {
                                    mFSL.onDataSended(per);
                                }
                            });
                        }
                        continue;
                    }
                    raFile.seek((long)(p * (long)mBlockSize));

                    final int rs = raFile.read(buffer);
                    Data_SendFileDat data;
                    if (rs == mBlockSize) {
                        data = new Data_SendFileDat(p, buffer);
                    } else if (rs > 0){
                        final byte[] fitBuffer = new byte[rs]; 
                        System.arraycopy(buffer, 0, fitBuffer, 0, rs);
                        data = new Data_SendFileDat(p, fitBuffer);
                    } else {
                        break;
                    }
                    data.setKey(mDataKey);
                    DataPackage pkg = DataPackageFactory.peekDataPackage(DataPackage.PKG_TYPE_BYTE_BUFFER);
                    pkg.setAction(DataPackage.ACTION_INFORM);
                    pkg.setByteBuffer(data.toBytes());

                    nm.sendPackage(pkg, mTarget);

                    // Touch the receiver.
                    final NetDataReceiverManager ndrm = nm.getNetDataReceiverManager();
                    ndrm.touch(DataPackage.ACTION_REPLY, NetData.ID_FILE_TRANSCEIVE_ACK, mCmdKey);
                    ndrm.touch(DataPackage.ACTION_REPLY, NetData.ID_FILE_TRANSCEIVE_ERROR, mCmdKey);
                } catch (IOException e) {
                    e.printStackTrace();
                    if (mFSL != null) {
                        final Handler h = new Handler(Looper.getMainLooper());
                        h.post(new Runnable() {
                            public void run() {
                                mFSL.onError(FDTManager.ERROR_CODE_LOCAL_IO);
                            }
                        });
                    }
                    break;
                }

                i++;
                final float per = (float)i / (float)mBlockIndexes.length;
                if (mFSL != null) {
                    final Handler h = new Handler(Looper.getMainLooper());
                    h.post(new Runnable() {
                        public void run() {
                            mFSL.onDataSended(per);
                        }
                    });
                }

                // Make sure the thread do not always occupy the CPU.
                try {
                    Thread.sleep(1);
                } catch (InterruptedException e) {
                    Log.w(TAG, "The thread is interrupted!");
                    break;
                }
            }
        } catch (OutOfMemoryError e) {
            // No Memory, just cancel this operation.
            e.printStackTrace();
        }

        try {
            raFile.close();
        } catch (IOException e) {
            e.printStackTrace();
        }
    }
}
