package com.oozic.time;

import java.util.Timer;
import java.util.TimerTask;

import android.os.Handler;

public class TimeLine {
    /**
     * LCD FPS
     */
    public static final int LCD_FPS = 18;

    /**
     * Normal FPS
     */
    public static final int NORMAL_FPS = 30;

    /**
     * High FPS
     */
    public static final int HIGH_FPS = 60;

    /**
     * Time line timer;
     */
    private Timer mTL;
    
    /**
     * Time line task;
     */
    private TimeLineTask mTLT;

    /**
     * ANDROID OS handler used to post runnable to main thread.
     */
    Handler mH;

    /**
     * It is the true if the time value goes forward. And it is false if the time value goes
     * reward.
     */
    private boolean mForward;

    /**
     * Constructs the time line.
     */
    public TimeLine() {
        this(true);
    }

    /**
     * It constructs the main thread.
     * @param runInMainThread It decides if the call back runnable run in the main thread.
     */
    public TimeLine(boolean runInMainThread) {
        // Initialize the variables.
        mTL = null;
        mTLT = null;
        if (runInMainThread)
            mH = new Handler();
        else
            mH = null;
    }

    /**
     * Start a time line.
     * @param begin Begin value of time line.
     * @param end End value of time line.
     * @param duration Duration in millisecond unit.
     * @param fps Frame per second. 
     * @param onActive The runnable runs when time line task is active.
     * @param onFinish The runnable runs when time line task is finished.
     */
    public void start(final float begin, final float end,
            final int duration, final int fps,
            final Runnable onActive, final Runnable onFinish) {
        int f, d;
        float b = begin;

        if (fps > HIGH_FPS) f = HIGH_FPS;
        else if (fps <= 0) f = 1;
        else f = fps;

        if (duration < 0) d = 0;
        else d = duration;
        
        if (mTLT != null) {
            mTL.cancel();
            mTLT.cancel();
            if (begin < end) {
                if (begin < mTLT.getValue()) b = mTLT.getValue();
            } else {
                if (begin > mTLT.getValue()) b = mTLT.getValue();
            }
        }

        mTL = new Timer();
        if (b <= end) mForward = true;
        else mForward = false;
        mTLT = new TimeLineTask(b, end, d, f, onActive, onFinish);
        mTL.scheduleAtFixedRate(mTLT, 0, 1000 / f);
    }

    /**
     * It checks if the time value goes forward.
     * @return It returns true if the time value goes forward. And it return false otherwise.
     */
    public boolean isForward() {
        return mForward;
    }

    /**
     * Clear the time line.
     */
    public void clear() {
        if (mTLT != null) {
            mTL.cancel();
            mTLT.cancel();
            mTL = null;
            mTLT = null;
        }
    }

    /**
     * Get current time line value
     * @return It returns value range from 0 to 1 when it can get the value. It returns -1 when it
     * cannot get the time line value.
     */
    public float getValue() {
        if (mTLT != null) {
            return mTLT.getValue();
        }
        return -1;
    }

    private class TimeLineTask extends TimerTask {
        /**
         * Begin value, its value range is from 0.0f to 1.0f.
         */
        private final float mBegin;

        /**
         * End alpha value, its value range is from 0.0 to 1.0f
         */
        private final float mEnd;

        /**
         * Current alpha value, its value range is from 0.0 to 1.0f
         */
        private float mValue;

        /**
         * Delta change per frame;
         */
        private final float mDelta;

        /**
         * Call back function runs when time line is active.
         */
        final protected Runnable mOnActive;

        /**
         * Call back function runs when time line is finished.
         */
        final protected Runnable mOnFinish;

        /**
         * Constructor of this class
         * @param begin Begin value of time line.
         * @param end End value of time line.
         * @param duration Duration in millisecond unit.
         * @param fps Frame per second. 
         * @param onActive The runnable runs when time line task is active.
         * @param onFinish The runnable runs when time line task is finished.
         */
        public TimeLineTask(final float begin, final float end,
                final int duration, final int fps,
                final Runnable onActive, final Runnable onFinish) {
            if(begin > 1.0f) {
                mBegin = 1.0f;
            } else if (begin < 0.0f) {
                mBegin = 0.0f;
            } else {
                mBegin = begin;
            }
            
            if (end > 1.0f) {
                mEnd = 1.0f;
            } else if (end < 0.0f) {
                mEnd = 0.0f;
            } else {
                mEnd = end;
            }

            int fn = (int)(fps * duration / 1000 + 0.5);
            if (fn == 0) fn = 1;

            mDelta = (mEnd - mBegin) / fn;
            mValue = mBegin;

            mOnActive = onActive;
            mOnFinish = onFinish;
        }

        public void run() {
            mValue += mDelta;
            if (mEnd < mBegin) {
                if (mValue <= mEnd) {
                    // Stop timer here.
                    mValue = mEnd;
                    cancel();
                    if(mOnActive != null) {
                        if (mH != null) mH.post(mOnActive);
                        else mOnActive.run();
                    }
                    if (mOnFinish != null) {
                        if (mH != null) mH.post(mOnFinish);
                        else mOnFinish.run();
                    }
                    return;
                }
            } else {
                if (mValue >= mEnd) {
                    // Stop timer here.
                    mValue = mEnd;
                    cancel();
                    if(mOnActive != null) {
                        if (mH != null) mH.post(mOnActive);
                        else mOnActive.run();
                    }
                    if (mOnFinish != null) {
                        if (mH != null) mH.post(mOnFinish);
                        else mOnFinish.run();
                    }
                    return;
                }
            }

            if(mOnActive != null) {
                if (mH != null) mH.post(mOnActive);
                else mOnActive.run();
            }
        }

        /**
         * This method help to get the alpha value.
         * @return It returns current value of time line.
         */
        public float getValue() {
            return mValue;
        }
    }
}
