package com.oozic.view;

import com.oozic.time.TimeLine;

import android.content.Context;
import android.graphics.Canvas;
import android.graphics.Paint;
import android.graphics.Paint.Align;
import android.graphics.Paint.FontMetrics;
import android.graphics.Path;
import android.graphics.Paint.Style;
import android.view.View;

/**
 * It is the view show a horizontal transparent down cursor.
 * @author CHEN JIAN WEN
 *
 */
public class HorizontalTextCursorView extends View {

    /**
     * It is the paint to draw cursor
     */
    protected Paint mPaint;

    /**
     * It is the path to draw.
     */
    protected Path mPath;

    /**
     * It is the target cursor.
     */
    protected int mTargetCursor;

    /**
     * It is the current cursor.
     */
    protected int mCurrentCursor;

    /**
     * It is the difference between target cursor and current cursor.
     */
    protected float mCursorDifference;

    /**
     * It is the animation time line.
     */
    protected TimeLine mCursorAnimationTL;

    /**
     * It is the width of the view.
     */
    protected int mWidth;

    /**
     * It is the height of the view.
     */
    protected int mHeight;

    /**
     * It is the text baseline.
     */
    protected int mBaseLine;

    /**
     * It is the text to show.
     */
    protected String mText = null;

    /**
     * It is the process to implement the cursor animation.
     * <B>Note</B>, this process should be given value from 0 to 1.
     */
    protected final Runnable mCursorAnimationOnActive = new Runnable() {
            @Override
            public void run() {
                final float v = mCursorAnimationTL.getValue();
                final float av = 1.0f - v;
                mCursorDifference *= av;
                mCurrentCursor = (int)(mTargetCursor - mCursorDifference);
                postInvalidate();
            }
        };

    /**
     * It constructs the view.
     * @param context It is the system context.
     */
    public HorizontalTextCursorView(Context context) {
        super(context);

        mPaint = new Paint();
        mPaint.setStyle(Style.FILL_AND_STROKE);
        mPaint.setColor(0xFFFFFFFF);
        mPaint.setStrokeWidth(0.5f);
        mPaint.setTextAlign(Align.CENTER);
        mPaint.setAntiAlias(true);
        mPaint.setShadowLayer(3, 2, 2, 0xFF000000);
        mTargetCursor = -100;
        mCurrentCursor = -100;
        mCursorDifference = 0.0f;
        mCursorAnimationTL = new TimeLine(false);
    }

    @Override
    protected void onSizeChanged(int w, int h, int oldw, int oldh) {
        super.onSizeChanged(w, h, oldw, oldh);
        mWidth = w;
        mHeight = h;
        //preparePath(w, h);
        mPaint.setTextSize(h * 0.8f);
        final FontMetrics fm = mPaint.getFontMetrics();
        mBaseLine = (int)-fm.top;
    }

    @Override
    protected void onDraw(Canvas canvas) {
        if (mText != null)
            canvas.drawText(mText, mCurrentCursor, mBaseLine, mPaint);
    }

    /**
     * It sets the cursor position.
     * @param x It is the position in x axis.
     * @apram text It is the text to change.
     */
    public void setCursorPosition(int x, String text) {
        mTargetCursor = x;
        mText = text;
        startCursorAnimation();
    }

    /**
     * It starts the cursor aniamtion.
     */
    private void startCursorAnimation() {
        mCursorAnimationTL.clear();
        mCursorDifference = mTargetCursor - mCurrentCursor;
        int duration = Math.abs((int)(mCursorDifference * 2));
        if (duration > 1000) duration = 1000;
        else if (duration < 300) duration = 300;
        mCursorAnimationTL.start(0, 1, duration, TimeLine.HIGH_FPS,
                mCursorAnimationOnActive, null);
    }

    /**
     * It sets the background color of cursor.
     * @param color It is the color to set.
     */
    public void setCursorBackgroundColor(int color) {
        if (mPaint != null) mPaint.setColor(color);
    }
}
